from __future__ import annotations

import logging
import math
import statistics
import time

import cv2
import numpy as np
from PyQt5 import QtCore
from PyQt5.QtGui import QBrush, QDoubleValidator, QImage, QPen, QPixmap
from PyQt5.QtWidgets import (
    QGraphicsPixmapItem,
    QGraphicsScene,
    QGraphicsView,
    QGridLayout,
    QHBoxLayout,
    QLabel,
    QLineEdit,
    QMessageBox,
    QPushButton,
    QWidget,
)

from module_qc_nonelec_gui.qc_tests.VISUAL_INSPECTION.functions.cv2_func import (
    img_cvt_rgb,
)

log = logging.getLogger(__name__)


class GraphicsScene(QGraphicsScene):
    def __init__(self, trimmer=None):
        QGraphicsScene.__init__(self, None)
        self.setSceneRect(0, 0, 0, 0)
        self.trimmer = trimmer
        self.turn = 0
        self.GA1_x = 0
        self.GA1_y = 0
        self.GA3_x = 0
        self.GA3_y = 0

    def mousePressEvent(self, event):
        x = event.scenePos().x()
        y = event.scenePos().y()

        if self.turn == 0:
            self.GA1_x = x
            self.GA1_y = y
            self.turn = 1

            self.addEllipse(
                self.GA1_x - 5,
                self.GA1_y - 5,
                10,
                10,
                QPen(QtCore.Qt.yellow),
                QBrush(QtCore.Qt.yellow),
            )
        else:
            self.GA3_x = x
            self.GA3_y = y
            self.turn = 0

            self.addEllipse(
                self.GA3_x - 5,
                self.GA3_y - 5,
                10,
                10,
                QPen(QtCore.Qt.yellow),
                QBrush(QtCore.Qt.yellow),
            )

            time.sleep(1)

            self.trimmer.trim_image_manual(
                {
                    "GA1": {"x": self.GA1_x, "y": self.GA1_y},
                    "GA3": {"x": self.GA3_x, "y": self.GA3_y},
                }
            )


class Trimmer(QWidget):
    def __init__(self, parent, custom_params):
        super(QWidget, self).__init__(parent)
        self.parent = parent
        self.cropped = None
        self.image = None
        self.item = None

        log.info(f"Trimmer.__init__(): custom_params = {custom_params}")

        self.nLayoutRows = 0

        self.setupCommon1()
        self.setupCustom(custom_params)
        self.setupCommon2()

        self.tmp_cropped_path = f"{self.parent.temp_dir_path}/tmp_cropped.jpg"
        self.tmp_preview_path = f"{self.parent.temp_dir_path}/tmp_preview.jpg"

        self.trim()

    def setupCommon1(self):
        self.layout = QGridLayout()

        # scene for module picture
        self.view = QGraphicsView()
        self.scene = GraphicsScene(self)
        self.srect = self.view.rect()
        self.view.setScene(self.scene)

        # load module picture
        img_org = cv2.imread(self.parent.origImgPath[self.parent.mode])
        h, w, d = img_org.shape
        scale = min(600.0 / float(h), 600.0 / float(w))

        resize_img = cv2.resize(
            img_org, dsize=None, fx=scale, fy=scale, interpolation=cv2.INTER_LANCZOS4
        )

        img_rgb, h_rgb, w_rgb, d_rgb = img_cvt_rgb(resize_img)
        bytesPerLine = d_rgb * w_rgb

        self.image = QImage(
            img_rgb.data, w_rgb, h_rgb, bytesPerLine, QImage.Format_RGB888
        )
        self.item = QGraphicsPixmapItem(QPixmap.fromImage(self.image))
        self.scene.addItem(self.item)
        self.view.setScene(self.scene)
        self.view.setFixedWidth(650)
        self.view.setFixedHeight(650)

        self.layout.addWidget(self.view, self.nLayoutRows, 0, 1, 2)
        self.nLayoutRows = self.nLayoutRows + 1

        self.buttons = QHBoxLayout()

    def setupCustom(self, custom_params):
        pass

    def setupCommon2(self):
        button_back = QPushButton("Back")
        button_back.clicked.connect(self.parent.load_img)

        button_rotate = QPushButton("Rotate")
        button_rotate.clicked.connect(self.rotate)

        button_crop = QPushButton("Preview")
        button_crop.clicked.connect(self.trim)

        button_ok = QPushButton("Next")
        button_ok.clicked.connect(self.go_next)

        self.buttons.addWidget(button_back)
        self.buttons.addWidget(button_rotate)
        self.buttons.addWidget(button_crop)
        self.buttons.addWidget(button_ok)

        self.layout.addLayout(self.buttons, self.nLayoutRows, 0)
        self.nLayoutRows = self.nLayoutRows + 1
        self.setLayout(self.layout)

    def rotate(self):
        self.parent.rotate_img()
        self.trim()

    def trim(self):
        try:
            self.trim_image(self.getParameters())

        except Exception as e:
            log.exception(e)

            QMessageBox.warning(
                self,
                "Warning",
                "Failed in cropping!\n\nManually click the center of the GA1 Pickup Point, and then click the center of the GA3 Pickup Point to determine the frame",
            )

        preview = cv2.imread(self.tmp_preview_path)

        self.cropped = cv2.imread(self.tmp_cropped_path)

        h, w, d = preview.shape

        scale = min(600.0 / float(h), 600.0 / float(w))

        resize_img = cv2.resize(
            preview, dsize=None, fx=scale, fy=scale, interpolation=cv2.INTER_LANCZOS4
        )

        img_rgb, h_rgb, w_rgb, d_rgb = img_cvt_rgb(resize_img)
        bytesPerLine = d_rgb * w_rgb

        self.image = QImage(
            img_rgb.data, w_rgb, h_rgb, bytesPerLine, QImage.Format_RGB888
        )

        self.view.items().clear()
        self.scene.clear()

        pixmap = QPixmap.fromImage(self.image)
        self.item = QGraphicsPixmapItem(pixmap)
        self.scene.addItem(self.item)
        self.view.setScene(self.scene)

    def trim_image(self, params):
        log.info(params)

    def getParameters(self):
        return {}

    def go_next(self):
        self.parent.update_img(self.cropped)
        self.parent.split_img()


#################################################################################################


class QuadFrontTrimmer(Trimmer):
    def setupCustom(self, custom_params):
        self.cropRange = custom_params.get("CropRange", 21.8)
        log.info(f"setupCustom(): Custom Crop Range = {self.cropRange} mm")

        self.handles = QHBoxLayout()
        self.handles2 = QHBoxLayout()

        label_rot = QLabel(self)
        label_rot.setText("Rotation: [-3.00, 3.00]")

        label_brightness = QLabel(self)
        label_brightness.setText("Brightness: [-100, 100]")

        label_contrast = QLabel(self)
        label_contrast.setText("Contrast: [0, 2]")

        label_blur = QLabel(self)
        label_blur.setText("Blur: [3, 10]")

        label_tweak = QLabel(self)
        label_tweak.setText("Tweak: [20, 50]")

        self.input_rot = QLineEdit(self)
        self.input_rot.setFixedWidth(40)
        self.input_rot.setValidator(QDoubleValidator(-3.0, 3.0, 2))
        self.input_rot.setText(str(0.00))

        self.input_brightness = QLineEdit(self)
        self.input_brightness.setFixedWidth(40)
        self.input_brightness.setValidator(QDoubleValidator(-100, 100, 0))
        self.input_brightness.setText(str(custom_params.get("Brightness", -20)))

        self.input_contrast = QLineEdit(self)
        self.input_contrast.setFixedWidth(40)
        self.input_contrast.setValidator(QDoubleValidator(0.0, 2.0, 1))
        self.input_contrast.setText(str(custom_params.get("Contrast", 1.3)))

        self.input_blur = QLineEdit(self)
        self.input_blur.setFixedWidth(40)
        self.input_blur.setValidator(QDoubleValidator(1, 5, 0))
        self.input_blur.setText(str(custom_params.get("InitialBlur", 5)))

        self.input_tweak = QLineEdit(self)
        self.input_tweak.setFixedWidth(40)
        self.input_tweak.setValidator(QDoubleValidator(20, 50, 0))
        self.input_tweak.setText(str(custom_params.get("InitialParam2", 35)))

        self.handles.addWidget(label_rot)
        self.handles.addWidget(self.input_rot)
        self.handles.addWidget(label_brightness)
        self.handles.addWidget(self.input_brightness)
        self.handles.addWidget(label_contrast)
        self.handles.addWidget(self.input_contrast)
        self.handles.addStretch()

        self.handles2.addWidget(label_blur)
        self.handles2.addWidget(self.input_blur)
        self.handles2.addWidget(label_tweak)
        self.handles2.addWidget(self.input_tweak)
        self.handles2.addStretch()

        self.layout.addLayout(self.handles, self.nLayoutRows, 0)
        self.nLayoutRows = self.nLayoutRows + 1

        self.layout.addLayout(self.handles2, self.nLayoutRows, 0)
        self.nLayoutRows = self.nLayoutRows + 1

    def getParameters(self):
        # get the original image
        rot_angle = float(self.input_rot.text())
        init_blur = int(self.input_blur.text())
        contrast = float(self.input_contrast.text())
        brightness = float(self.input_brightness.text())
        init_param2 = int(self.input_tweak.text())

        log.info(
            f"cropping with rot_angle {rot_angle}, init_blur {init_blur}, contrast {contrast}, brightness {brightness}, init_param2 {init_param2}"
        )

        return {
            "rot_angle": rot_angle,
            "init_blur": init_blur,
            "contrast": contrast,
            "brightness": brightness,
            "init_param2": init_param2,
        }

    def trim_image(self, params):
        rot_angle = params["rot_angle"]
        init_blur = params["init_blur"]
        contrast = params["contrast"]
        brightness = params["brightness"]
        init_param2 = params["init_param2"]

        image_1 = self.parent.img_bgr.copy()

        image_center = tuple(np.array(image_1.shape[1::-1]) / 2)
        rot_mat = cv2.getRotationMatrix2D(image_center, rot_angle, 1.0)
        image_2 = cv2.warpAffine(
            image_1, rot_mat, image_1.shape[1::-1], flags=cv2.INTER_LINEAR
        )

        image_3 = cv2.convertScaleAbs(image_2, alpha=contrast, beta=brightness)

        height_org, width_org, channel = image_3.shape

        resize_scale = (
            1200 / width_org if width_org >= height_org else 1200 / height_org
        )

        image_4 = cv2.resize(
            image_3, None, None, resize_scale, resize_scale, cv2.INTER_LANCZOS4
        )

        height, width, channel = image_4.shape

        for i in range(height):
            for j in range(width):
                image_3.itemset((i, j, 1), 0)

        gray = cv2.cvtColor(image_4, cv2.COLOR_BGR2GRAY)
        cv2.imwrite(self.tmp_preview_path, gray)

        image_inverted = 255 - gray

        cv2.imwrite(
            self.tmp_preview_path.replace("preview", "preview_gray_inverted"),
            image_inverted,
        )

        log.info("Searching pickup points...")

        gauge_dic = None
        pickup_points = None

        for blurSize in range(init_blur, 3, -1):
            try:
                gauge_dic, pickup_points = self.houghScan(
                    image_inverted, blurSize, init_param2
                )
                break
            except Exception as e:
                log.info(str(e))
                continue

        if not gauge_dic:
            msg = "failed to find any pickup points"
            raise Exception(msg)

        gauge_std = sum(g["length"] for g in gauge_dic) / len(gauge_dic)
        center_x = int(sum(p[0] for p in pickup_points) / len(pickup_points))
        center_y = int(sum(p[1] for p in pickup_points) / len(pickup_points))

        pitch = gauge_std / 20.0  # pixels / mm

        log.info(
            f"gauge_std = {gauge_std:.3f} px: 1mm = {pitch:.3f} px, center = ( {center_x:6.3f}, {center_y:6.3f} )"
        )

        cropPitch = self.cropRange * pitch
        x_remaining_left = center_x - cropPitch
        x_remaining_right = width - (center_x + cropPitch)
        if x_remaining_left > x_remaining_right:
            # module shifted on the picture right side
            x_shift = width - center_x if x_remaining_right <= 0 else cropPitch

        else:
            x_shift = center_x if x_remaining_left <= 0 else cropPitch

        y_remaining_top = center_y - cropPitch
        y_remaining_bottom = height - (center_y + cropPitch)
        if y_remaining_top > y_remaining_bottom:
            # module shifted on the picture bottom side
            y_shift = height - center_y if y_remaining_bottom <= 0 else cropPitch

        else:
            y_shift = center_y if y_remaining_top <= 0 else cropPitch

        x_edge1_0 = int(center_x - x_shift)
        y_edge1_0 = int(center_y - y_shift)
        x_edge2_0 = int(center_x + x_shift)
        y_edge2_0 = int(center_y + y_shift)

        x_edge1 = int(x_edge1_0 / resize_scale)
        y_edge1 = int(y_edge1_0 / resize_scale)
        x_edge2 = int(x_edge2_0 / resize_scale)
        y_edge2 = int(y_edge2_0 / resize_scale)

        crop_info = [[x_edge1, x_edge2], [y_edge1, y_edge2]]
        log.info(f"crop_info = {crop_info}")

        cropped = image_2[y_edge1:y_edge2, x_edge1:x_edge2].copy()

        cv2.imwrite(self.tmp_cropped_path, cropped)

        marker_width = int(0.005 * width)

        color = (0, 255, 128)

        # Draw the center reticle
        gray_bgr = cv2.cvtColor(gray, cv2.COLOR_GRAY2RGB)

        cv2.line(
            gray_bgr,
            (center_x - 20, center_y),
            (center_x + 20, center_y),
            color,
            marker_width,
        )
        cv2.line(
            gray_bgr,
            (center_x, center_y - 20),
            (center_x, center_y + 20),
            color,
            marker_width,
        )

        cv2.line(
            gray_bgr,
            (x_edge1_0, y_edge1_0),
            (x_edge1_0 + 50, y_edge1_0),
            color,
            marker_width,
        )
        cv2.line(
            gray_bgr,
            (x_edge1_0, y_edge1_0),
            (x_edge1_0, y_edge1_0 + 50),
            color,
            marker_width,
        )

        cv2.line(
            gray_bgr,
            (x_edge2_0, y_edge1_0),
            (x_edge2_0 - 50, y_edge1_0),
            color,
            marker_width,
        )
        cv2.line(
            gray_bgr,
            (x_edge2_0, y_edge1_0),
            (x_edge2_0, y_edge1_0 + 50),
            color,
            marker_width,
        )

        cv2.line(
            gray_bgr,
            (x_edge1_0, y_edge2_0),
            (x_edge1_0 + 50, y_edge2_0),
            color,
            marker_width,
        )
        cv2.line(
            gray_bgr,
            (x_edge1_0, y_edge2_0),
            (x_edge1_0, y_edge2_0 - 50),
            color,
            marker_width,
        )

        cv2.line(
            gray_bgr,
            (x_edge2_0, y_edge2_0),
            (x_edge2_0 - 50, y_edge2_0),
            color,
            marker_width,
        )
        cv2.line(
            gray_bgr,
            (x_edge2_0, y_edge2_0),
            (x_edge2_0, y_edge2_0 - 50),
            color,
            marker_width,
        )

        for p in pickup_points:
            cv2.circle(
                gray_bgr,
                (int(p[0]), int(p[1])),
                int(p[2]),
                color,
                int(marker_width * 0.5),
            )

        cv2.imwrite(self.tmp_preview_path, gray_bgr)

        return crop_info

    def trim_image_manual(self, manual_points):
        params = self.getParameters()

        # rot_angle = params["rot_angle"]
        # init_blur = params["init_blur"]
        contrast = params["contrast"]
        brightness = params["brightness"]
        # init_param2 = params["init_param2"]

        image_1 = self.parent.img_bgr.copy()

        image_center = tuple(np.array(image_1.shape[1::-1]) / 2)
        rot_mat = cv2.getRotationMatrix2D(image_center, 0.0, 1.0)
        image_2 = cv2.warpAffine(
            image_1, rot_mat, image_1.shape[1::-1], flags=cv2.INTER_LINEAR
        )

        image_3 = cv2.convertScaleAbs(image_2, alpha=contrast, beta=brightness)

        height_org, width_org, channel = image_3.shape

        resize_scale = (
            1200 / width_org if width_org >= height_org else 1200 / height_org
        )

        image_4 = cv2.resize(
            image_3, None, None, resize_scale, resize_scale, cv2.INTER_LANCZOS4
        )

        height, width, channel = image_4.shape

        for i in range(height):
            for j in range(width):
                image_3.itemset((i, j, 1), 0)

        gray = cv2.cvtColor(image_4, cv2.COLOR_BGR2GRAY)
        cv2.imwrite(self.tmp_preview_path, gray)

        image_inverted = 255 - gray

        cv2.imwrite(
            self.tmp_preview_path.replace("preview", "preview_gray_inverted"),
            image_inverted,
        )

        log.info("Searching pickup points...")

        center_x = int(
            sum([manual_points.get("GA1").get("x"), manual_points.get("GA3").get("x")])
        )
        center_y = int(
            sum([manual_points.get("GA1").get("y"), manual_points.get("GA3").get("y")])
        )

        gauge_std = sum(
            [
                abs(
                    manual_points.get("GA1").get("x")
                    - manual_points.get("GA3").get("x")
                ),
                abs(
                    manual_points.get("GA1").get("y")
                    - manual_points.get("GA3").get("y")
                ),
            ]
        )

        pitch = gauge_std / 20.0  # pixels / mm

        log.info(
            f"gauge_std = {gauge_std:.3f} px: 1mm = {pitch:.3f} px, center = ( {center_x:6.3f}, {center_y:6.3f} )"
        )

        x_edge1_0 = max(int(center_x - self.cropRange * pitch), 0)
        y_edge1_0 = max(int(center_y - self.cropRange * pitch), 0)
        x_edge2_0 = min(int(center_x + self.cropRange * pitch), width - 1)
        y_edge2_0 = min(int(center_y + self.cropRange * pitch), height - 1)

        x_edge1 = max(int((center_x - self.cropRange * pitch) / resize_scale), 0)
        y_edge1 = max(int((center_y - self.cropRange * pitch) / resize_scale), 0)
        x_edge2 = min(
            int((center_x + self.cropRange * pitch) / resize_scale), width_org - 1
        )
        y_edge2 = min(
            int((center_y + self.cropRange * pitch) / resize_scale), height_org - 1
        )

        crop_info = [[x_edge1, x_edge2], [y_edge1, y_edge2]]
        log.info(f"crop_info = {crop_info}")

        cropped = image_2[y_edge1:y_edge2, x_edge1:x_edge2].copy()

        cv2.imwrite(self.tmp_cropped_path, cropped)

        marker_width = int(0.005 * width)

        color = (0, 255, 128)

        # Draw the center reticle
        gray_bgr = cv2.cvtColor(gray, cv2.COLOR_GRAY2RGB)

        cv2.line(
            gray_bgr,
            (center_x - 20, center_y),
            (center_x + 20, center_y),
            color,
            marker_width,
        )
        cv2.line(
            gray_bgr,
            (center_x, center_y - 20),
            (center_x, center_y + 20),
            color,
            marker_width,
        )

        cv2.line(
            gray_bgr,
            (x_edge1_0, y_edge1_0),
            (x_edge1_0 + 50, y_edge1_0),
            color,
            marker_width,
        )
        cv2.line(
            gray_bgr,
            (x_edge1_0, y_edge1_0),
            (x_edge1_0, y_edge1_0 + 50),
            color,
            marker_width,
        )

        cv2.line(
            gray_bgr,
            (x_edge2_0, y_edge1_0),
            (x_edge2_0 - 50, y_edge1_0),
            color,
            marker_width,
        )
        cv2.line(
            gray_bgr,
            (x_edge2_0, y_edge1_0),
            (x_edge2_0, y_edge1_0 + 50),
            color,
            marker_width,
        )

        cv2.line(
            gray_bgr,
            (x_edge1_0, y_edge2_0),
            (x_edge1_0 + 50, y_edge2_0),
            color,
            marker_width,
        )
        cv2.line(
            gray_bgr,
            (x_edge1_0, y_edge2_0),
            (x_edge1_0, y_edge2_0 - 50),
            color,
            marker_width,
        )

        cv2.line(
            gray_bgr,
            (x_edge2_0, y_edge2_0),
            (x_edge2_0 - 50, y_edge2_0),
            color,
            marker_width,
        )
        cv2.line(
            gray_bgr,
            (x_edge2_0, y_edge2_0),
            (x_edge2_0, y_edge2_0 - 50),
            color,
            marker_width,
        )

        cv2.circle(
            gray_bgr,
            (
                int(manual_points.get("GA1").get("x")) * 2,
                int(manual_points.get("GA1").get("y")) * 2,
            ),
            10,
            color,
            int(marker_width * 0.5),
        )

        cv2.circle(
            gray_bgr,
            (
                int(manual_points.get("GA3").get("x")) * 2,
                int(manual_points.get("GA3").get("y")) * 2,
            ),
            10,
            color,
            int(marker_width * 0.5),
        )

        cv2.imwrite(self.tmp_preview_path, gray_bgr)

        preview = cv2.imread(self.tmp_preview_path)

        self.cropped = cv2.imread(self.tmp_cropped_path)

        h, w, d = preview.shape

        scale = min(600.0 / float(h), 600.0 / float(w))

        resize_img = cv2.resize(
            preview, dsize=None, fx=scale, fy=scale, interpolation=cv2.INTER_LANCZOS4
        )

        img_rgb, h_rgb, w_rgb, d_rgb = img_cvt_rgb(resize_img)
        bytesPerLine = d_rgb * w_rgb

        self.image = QImage(
            img_rgb.data, w_rgb, h_rgb, bytesPerLine, QImage.Format_RGB888
        )

        self.view.items().clear()
        self.scene.clear()

        pixmap = QPixmap.fromImage(self.image)
        self.item = QGraphicsPixmapItem(pixmap)
        self.scene.addItem(self.item)
        self.view.setScene(self.scene)

        return crop_info

    def houghScan(self, image_inverted, blurSize, param2):
        log.info(f"  Hough search: blurSize = {blurSize}")

        blur = cv2.blur(image_inverted, (blurSize, blurSize))
        cv2.imwrite(
            self.tmp_preview_path.replace("preview", f"preview_blur_{blurSize}"), blur
        )

        height, width = image_inverted.shape

        iter_num = 0

        while True:
            log.info(f"  Hough search: blurSize = {blurSize}, param2 = {param2}")
            ret, image_4 = cv2.threshold(
                blur, thresh=120, maxval=255, type=cv2.THRESH_TOZERO
            )

            # detect circle
            circles = cv2.HoughCircles(
                image=image_4,
                method=cv2.HOUGH_GRADIENT,
                dp=1.0,
                minDist=min(width, height) * 0.12,
                param1=200,
                param2=param2,
                minRadius=int(min(width, height) * 0.03),
                maxRadius=int(min(width, height) * 0.10),
            )

            pickup_cands = []

            for i in circles[0, :]:
                # check if center is in middle of picture
                if (
                    i[0] < width * 0.05
                    or i[0] > width * 0.95
                    or i[1] < height * 0.05
                    or i[1] > height * 0.95
                ):
                    continue

                # draw the outer circle
                cv2.circle(image_4, (int(i[0]), int(i[1])), int(i[2]), (0, 0, 0), 2)

                # draw the center of the circle
                cv2.circle(image_4, (int(i[0]), int(i[1])), 2, (0, 0, 0), 3)

                pickup_cands += [i]

            gauge_dic = []
            pickup_points = set()

            for i, ci in enumerate(pickup_cands):
                for j in range(i + 1, len(pickup_cands)):
                    cj = pickup_cands[j]

                    dx = abs(cj[0] - ci[0])
                    dy = abs(cj[1] - ci[1])

                    ang = math.atan(dy / (dx + 1.0e-7))
                    deg = math.atan(1) / 45.0
                    length = math.hypot(dx, dy)

                    isHorizontal = ang < 2 * deg
                    isVertical = ang > 88 * deg

                    if not (isHorizontal or isVertical):
                        # log.info( f'{length:.2f} px, {ang/deg:6.3f} deg ==> skipped' )
                        continue

                    # log.info( f'    {"Horizontal" if isHorizontal else "Vertical":12s}: {length:.2f} px, {ang/deg:6.3f} deg' )
                    cv2.line(
                        image_4,
                        (int(ci[0]), int(ci[1])),
                        (int(cj[0]), int(cj[1])),
                        (0, 0, 0),
                        1,
                    )
                    gauge_dic += [
                        {
                            "horizontal": isHorizontal,
                            "vertical": isVertical,
                            "length": length,
                            "angle": ang / deg,
                        }
                    ]

                    for p in [ci, cj]:
                        pickup_points.add((p[0], p[1], p[2]))

            if len(pickup_points) < 4:
                param2 -= 1

            elif len(pickup_points) > 4:
                param2 += 1

            else:
                break

            iter_num += 1

            if iter_num > 50:
                msg = "failure in detection: too long iterations"
                log.info(msg)
                raise Exception(msg)

            if param2 > 80:
                msg = "failure in detection: scanned all param2 range"
                log.info(msg)
                raise Exception(msg)

        if len(gauge_dic) != 4:
            msg = "failure in detection: did not find good gauges"
            log.info(msg)
            raise Exception(msg)

        log.info("")
        log.info("------------------")
        log.info("Detected pickup points:")
        for p in pickup_points:
            log.info(
                f"    Pickup point: center: ({p[0]:.2f}, {p[1]:.2f}), radius: {p[2]:.2f}"
            )

        log.info("Detected pickup points gauges:")
        for g in gauge_dic:
            log.info(
                f'    {"Horizontal" if g["horizontal"] else "Vertical":12s}: length = {g["length"]:7.2f} pixels, angle = {g["angle"]:6.3f} deg'
            )

        return gauge_dic, pickup_points


#################################################################################################


class QuadBackTrimmer(Trimmer):
    def setupCustom(self, custom_params):
        self.cropRange = custom_params.get("CropRange", 21.8)
        log.info(
            f"QuadBackTrimmer.setupCustom(): Custom Crop Range = {self.cropRange} mm"
        )

        self.handles = QHBoxLayout()
        self.handles2 = QHBoxLayout()

        label_rot = QLabel(self)
        label_rot.setText("Rotation: [-3.00, 3.00]")

        label_brightness = QLabel(self)
        label_brightness.setText("Brightness: [-100, 100]")

        label_contrast = QLabel(self)
        label_contrast.setText("Contrast: [0, 4]")

        label_blur = QLabel(self)
        label_blur.setText("Blur: [1, 5]")

        label_cannyThr1 = QLabel(self)
        label_cannyThr1.setText("Thr1: [0, 255]")

        label_cannyThr2 = QLabel(self)
        label_cannyThr2.setText("Thr2: [0, 255]")

        self.input_rot = QLineEdit(self)
        self.input_rot.setFixedWidth(40)
        self.input_rot.setValidator(QDoubleValidator(-3.0, 3.0, 2))
        self.input_rot.setText(str(0.00))

        self.input_brightness = QLineEdit(self)
        self.input_brightness.setFixedWidth(40)
        self.input_brightness.setValidator(QDoubleValidator(-100, 100, 0))
        self.input_brightness.setText(str(custom_params.get("Brightness", 50)))

        self.input_contrast = QLineEdit(self)
        self.input_contrast.setFixedWidth(40)
        self.input_contrast.setValidator(QDoubleValidator(0.0, 4.0, 1))
        self.input_contrast.setText(str(custom_params.get("Contrast", 3.0)))

        self.input_blur = QLineEdit(self)
        self.input_blur.setFixedWidth(40)
        self.input_blur.setValidator(QDoubleValidator(1, 5, 0))
        self.input_blur.setText(str(5))

        self.input_cannyThr1 = QLineEdit(self)
        self.input_cannyThr1.setFixedWidth(40)
        self.input_cannyThr1.setValidator(QDoubleValidator(0, 255, 0))
        self.input_cannyThr1.setText(str(custom_params.get("CannyThr1", 100)))

        self.input_cannyThr2 = QLineEdit(self)
        self.input_cannyThr2.setFixedWidth(40)
        self.input_cannyThr2.setValidator(QDoubleValidator(0, 255, 0))
        self.input_cannyThr2.setText(str(custom_params.get("CannyThr2", 20)))

        self.handles.addWidget(label_rot)
        self.handles.addWidget(self.input_rot)
        self.handles.addWidget(label_brightness)
        self.handles.addWidget(self.input_brightness)
        self.handles.addWidget(label_contrast)
        self.handles.addWidget(self.input_contrast)

        self.handles2.addWidget(label_blur)
        self.handles2.addWidget(self.input_blur)
        self.handles2.addWidget(label_cannyThr1)
        self.handles2.addWidget(self.input_cannyThr1)
        self.handles2.addWidget(label_cannyThr2)
        self.handles2.addWidget(self.input_cannyThr2)

        self.layout.addLayout(self.handles, self.nLayoutRows, 0)
        self.nLayoutRows = self.nLayoutRows + 1

        msg = QLineEdit(self)
        msg.setFixedWidth(700)
        msg.setReadOnly(True)
        msg.setText(
            "If auto-detection fails, you can manually specify the cropping region by clicking two diagonal corners of the FEs"
        )
        self.layout.addWidget(msg, self.nLayoutRows, 0)
        self.nLayoutRows = self.nLayoutRows + 1

        self.layout.addLayout(self.handles2, self.nLayoutRows, 0)
        self.nLayoutRows = self.nLayoutRows + 1

        self.handles.addStretch()
        self.handles2.addStretch()

    def getParameters(self):
        # get the original image
        rot_angle = float(self.input_rot.text())
        init_blur = int(self.input_blur.text())
        contrast = float(self.input_contrast.text())
        brightness = float(self.input_brightness.text())
        cannyThr1 = int(self.input_cannyThr1.text())
        cannyThr2 = int(self.input_cannyThr2.text())

        log.info(
            f"cropping with rot_angle {rot_angle}, init_blur {init_blur}, contrast {contrast}, brightness {brightness}, cannyThr [{cannyThr1}, {cannyThr2}]"
        )

        return {
            "rot_angle": rot_angle,
            "init_blur": init_blur,
            "contrast": contrast,
            "brightness": brightness,
            "cannyThr1": cannyThr1,
            "cannyThr2": cannyThr2,
        }

    def trim_image_pre(self, params):
        rot_angle = params["rot_angle"]
        contrast = params["contrast"]
        brightness = params["brightness"]

        image_1 = self.parent.img_bgr.copy()

        image_center = tuple(np.array(image_1.shape[1::-1]) / 2)
        rot_mat = cv2.getRotationMatrix2D(image_center, rot_angle, 1.0)
        image_2 = cv2.warpAffine(
            image_1, rot_mat, image_1.shape[1::-1], flags=cv2.INTER_LINEAR
        )

        image_3 = cv2.convertScaleAbs(image_2, alpha=contrast, beta=brightness)

        height_org, width_org, channel = image_3.shape
        log.info(f"original image: width {width_org}, height {height_org}")

        resize_scale = (
            1200 / width_org if width_org >= height_org else 1200 / height_org
        )
        log.info(f"resize_scale = {resize_scale}")

        image_4 = cv2.resize(
            image_3, None, None, resize_scale, resize_scale, cv2.INTER_LANCZOS4
        )

        height, width, channel = image_4.shape

        for i in range(height):
            for j in range(width):
                image_4.itemset((i, j, 1), 0)

        gray = cv2.cvtColor(image_4, cv2.COLOR_BGR2GRAY)
        cv2.imwrite(self.tmp_preview_path, gray)

        return image_2, gray, width_org, height_org, width, height

    def trim_image(self, params):
        original, gray, width_org, height_org, width, height = self.trim_image_pre(
            params
        )

        init_blur = params["init_blur"]

        blur = cv2.blur(gray, (init_blur, init_blur))

        cannyThr1 = params["cannyThr1"]
        cannyThr2 = params["cannyThr2"]

        edges = cv2.Canny(blur, cannyThr2, cannyThr1)

        rho = 1  # distance resolution in pixels of the Hough grid
        theta = (
            4.0 * math.atan(1.0) / 30.0
        )  # angular resolution in radians of the Hough grid
        threshold = 200  # minimum number of votes (intersections in Hough grid cell)
        min_line_length = 200  # minimum number of pixels making up a line
        max_line_gap = 1200  # maximum gap in pixels between connectable line segments

        lines = []

        # Run Hough on edge detected image
        # Output "lines" is an array containing endpoints of detected line segments
        lines = cv2.HoughLinesP(
            edges, rho, theta, threshold, np.array([]), min_line_length, max_line_gap
        )

        if len(lines) == 0:
            msg = "No edge lines were detected"
            raise Exception(msg)

        xEdges = []
        yEdges = []

        for line in lines:
            x1 = line[0][0]
            y1 = line[0][1]
            x2 = line[0][2]
            y2 = line[0][3]
            dx = abs(x1 - x2)
            dy = abs(y1 - y2)
            deg = math.atan(1) / 45.0
            ang = math.atan(dy / (dx + 1.0e-7))
            isHorizontal = ang < 2 * deg
            isVertical = ang > 88 * deg

            if not (isHorizontal or isVertical):
                continue

            if isVertical:
                xEdges += [x1]
            if isHorizontal:
                yEdges += [y1]

        xEdges.sort()
        yEdges.sort()

        if len(xEdges) == 0:
            msg = "No horizontal edge lines were detected"
            raise Exception(msg)

        if len(yEdges) == 0:
            msg = "No horizontal edge lines were detected"
            raise Exception(msg)

        crop_info = self.trim_image_post(
            gray, width, height, width_org, height_org, lines, xEdges, yEdges
        )

        # crop np.ndarray -- y first, x second!
        cropped = original[
            crop_info[1][0] : crop_info[1][1], crop_info[0][0] : crop_info[0][1]
        ].copy()

        cv2.imwrite(self.tmp_cropped_path, cropped)

        return crop_info

    def trim_image_manual(self, manual_points):
        params = self.getParameters()

        # rot_angle = params["rot_angle"]
        # init_blur = params["init_blur"]
        contrast = params["contrast"]
        # brightness = params["brightness"]
        # init_param2 = params["init_param2"]

        image_1 = self.parent.img_bgr.copy()

        image_center = tuple(np.array(image_1.shape[1::-1]) / 2)
        rot_mat = cv2.getRotationMatrix2D(image_center, 0.0, 1.0)
        image_2 = cv2.warpAffine(
            image_1, rot_mat, image_1.shape[1::-1], flags=cv2.INTER_LINEAR
        )

        # image_3 = cv2.convertScaleAbs(image_2, alpha=contrast, beta=brightness)
        image_3 = cv2.convertScaleAbs(image_2, alpha=contrast, beta=0)

        height_org, width_org, channel = image_3.shape

        resize_scale = (
            1200 / width_org if width_org >= height_org else 1200 / height_org
        )

        image_4 = cv2.resize(
            image_3, None, None, resize_scale, resize_scale, cv2.INTER_LANCZOS4
        )

        height, width, channel = image_4.shape

        for i in range(height):
            for j in range(width):
                image_3.itemset((i, j, 1), 0)

        gray = cv2.cvtColor(image_4, cv2.COLOR_BGR2GRAY)
        cv2.imwrite(self.tmp_preview_path, gray)

        image_inverted = 255 - gray

        cv2.imwrite(
            self.tmp_preview_path.replace("preview", "preview_gray_inverted"),
            image_inverted,
        )

        log.info("Searching pickup points...")

        center_x = int(
            sum([manual_points.get("GA1").get("x"), manual_points.get("GA3").get("x")])
        )
        center_y = int(
            sum([manual_points.get("GA1").get("y"), manual_points.get("GA3").get("y")])
        )

        gauge_std = sum(
            [
                abs(
                    manual_points.get("GA1").get("x")
                    - manual_points.get("GA3").get("x")
                ),
                abs(
                    manual_points.get("GA1").get("y")
                    - manual_points.get("GA3").get("y")
                ),
            ]
        )

        pitch = gauge_std / 41.236  # pixels / mm

        log.info(
            f"gauge_std = {gauge_std:.3f} px: 1mm = {pitch:.3f} px, center = ( {center_x:6.3f}, {center_y:6.3f} )"
        )

        x_edge1_0 = max(int(center_x - self.cropRange * pitch), 0)
        y_edge1_0 = max(int(center_y - self.cropRange * pitch), 0)
        x_edge2_0 = min(int(center_x + self.cropRange * pitch), width - 1)
        y_edge2_0 = min(int(center_y + self.cropRange * pitch), height - 1)

        x_edge1 = max(int((center_x - self.cropRange * pitch) / resize_scale), 0)
        y_edge1 = max(int((center_y - self.cropRange * pitch) / resize_scale), 0)
        x_edge2 = min(
            int((center_x + self.cropRange * pitch) / resize_scale), width_org - 1
        )
        y_edge2 = min(
            int((center_y + self.cropRange * pitch) / resize_scale), height_org - 1
        )

        crop_info = [[x_edge1, x_edge2], [y_edge1, y_edge2]]
        log.info(f"crop_info = {crop_info}")

        cropped = image_2[y_edge1:y_edge2, x_edge1:x_edge2].copy()

        cv2.imwrite(self.tmp_cropped_path, cropped)

        marker_width = int(0.005 * width)

        color = (0, 255, 128)

        # Draw the center reticle
        gray_bgr = cv2.cvtColor(gray, cv2.COLOR_GRAY2RGB)

        cv2.line(
            gray_bgr,
            (center_x - 20, center_y),
            (center_x + 20, center_y),
            color,
            marker_width,
        )
        cv2.line(
            gray_bgr,
            (center_x, center_y - 20),
            (center_x, center_y + 20),
            color,
            marker_width,
        )

        cv2.line(
            gray_bgr,
            (x_edge1_0, y_edge1_0),
            (x_edge1_0 + 50, y_edge1_0),
            color,
            marker_width,
        )
        cv2.line(
            gray_bgr,
            (x_edge1_0, y_edge1_0),
            (x_edge1_0, y_edge1_0 + 50),
            color,
            marker_width,
        )

        cv2.line(
            gray_bgr,
            (x_edge2_0, y_edge1_0),
            (x_edge2_0 - 50, y_edge1_0),
            color,
            marker_width,
        )
        cv2.line(
            gray_bgr,
            (x_edge2_0, y_edge1_0),
            (x_edge2_0, y_edge1_0 + 50),
            color,
            marker_width,
        )

        cv2.line(
            gray_bgr,
            (x_edge1_0, y_edge2_0),
            (x_edge1_0 + 50, y_edge2_0),
            color,
            marker_width,
        )
        cv2.line(
            gray_bgr,
            (x_edge1_0, y_edge2_0),
            (x_edge1_0, y_edge2_0 - 50),
            color,
            marker_width,
        )

        cv2.line(
            gray_bgr,
            (x_edge2_0, y_edge2_0),
            (x_edge2_0 - 50, y_edge2_0),
            color,
            marker_width,
        )
        cv2.line(
            gray_bgr,
            (x_edge2_0, y_edge2_0),
            (x_edge2_0, y_edge2_0 - 50),
            color,
            marker_width,
        )

        cv2.circle(
            gray_bgr,
            (
                int(manual_points.get("GA1").get("x")) * 2,
                int(manual_points.get("GA1").get("y")) * 2,
            ),
            10,
            color,
            int(marker_width * 0.5),
        )

        cv2.circle(
            gray_bgr,
            (
                int(manual_points.get("GA3").get("x")) * 2,
                int(manual_points.get("GA3").get("y")) * 2,
            ),
            10,
            color,
            int(marker_width * 0.5),
        )

        cv2.imwrite(self.tmp_preview_path, gray_bgr)

        preview = cv2.imread(self.tmp_preview_path)

        self.cropped = cv2.imread(self.tmp_cropped_path)

        h, w, d = preview.shape

        scale = min(600.0 / float(h), 600.0 / float(w))

        resize_img = cv2.resize(
            preview, dsize=None, fx=scale, fy=scale, interpolation=cv2.INTER_LANCZOS4
        )

        img_rgb, h_rgb, w_rgb, d_rgb = img_cvt_rgb(resize_img)
        bytesPerLine = d_rgb * w_rgb

        self.image = QImage(
            img_rgb.data, w_rgb, h_rgb, bytesPerLine, QImage.Format_RGB888
        )

        self.view.items().clear()
        self.scene.clear()

        pixmap = QPixmap.fromImage(self.image)
        self.item = QGraphicsPixmapItem(pixmap)
        self.scene.addItem(self.item)
        self.view.setScene(self.scene)

        return crop_info

    def trim_image_post(
        self,
        image,
        width,
        height,
        width_org,
        height_org,
        lines,
        xEdges,
        yEdges,
        std1=42.2,
        std2=41.4,
    ):
        center_x = int((xEdges[0] + xEdges[-1]) / 2.0)
        center_y = int((yEdges[0] + yEdges[-1]) / 2.0)

        horLength = abs(xEdges[0] - xEdges[-1])
        verLength = abs(yEdges[0] - yEdges[-1])
        pitch = (horLength / std1 + verLength / std2) / 2.0  # pixels / mm

        marker_width0 = int(0.005 * width)

        x_edge1_0 = max(int(center_x - self.cropRange * pitch), 0)
        y_edge1_0 = max(int(center_y - self.cropRange * pitch), 0)
        x_edge2_0 = min(int(center_x + self.cropRange * pitch), width - 1)
        y_edge2_0 = min(int(center_y + self.cropRange * pitch), height - 1)

        # Draw the center reticle
        gray_bgr = cv2.cvtColor(image, cv2.COLOR_GRAY2RGB)
        color = (0, 255, 128)

        cv2.line(
            gray_bgr,
            (center_x - 20, center_y),
            (center_x + 20, center_y),
            color,
            marker_width0,
        )
        cv2.line(
            gray_bgr,
            (center_x, center_y - 20),
            (center_x, center_y + 20),
            color,
            marker_width0,
        )

        cv2.line(
            gray_bgr,
            (x_edge1_0, y_edge1_0),
            (x_edge1_0 + 50, y_edge1_0),
            color,
            marker_width0,
        )
        cv2.line(
            gray_bgr,
            (x_edge1_0, y_edge1_0),
            (x_edge1_0, y_edge1_0 + 50),
            color,
            marker_width0,
        )

        cv2.line(
            gray_bgr,
            (x_edge2_0, y_edge1_0),
            (x_edge2_0 - 50, y_edge1_0),
            color,
            marker_width0,
        )
        cv2.line(
            gray_bgr,
            (x_edge2_0, y_edge1_0),
            (x_edge2_0, y_edge1_0 + 50),
            color,
            marker_width0,
        )

        cv2.line(
            gray_bgr,
            (x_edge1_0, y_edge2_0),
            (x_edge1_0 + 50, y_edge2_0),
            color,
            marker_width0,
        )
        cv2.line(
            gray_bgr,
            (x_edge1_0, y_edge2_0),
            (x_edge1_0, y_edge2_0 - 50),
            color,
            marker_width0,
        )

        cv2.line(
            gray_bgr,
            (x_edge2_0, y_edge2_0),
            (x_edge2_0 - 50, y_edge2_0),
            color,
            marker_width0,
        )
        cv2.line(
            gray_bgr,
            (x_edge2_0, y_edge2_0),
            (x_edge2_0, y_edge2_0 - 50),
            color,
            marker_width0,
        )

        for line in lines:
            x1 = line[0][0]
            y1 = line[0][1]
            x2 = line[0][2]
            y2 = line[0][3]

            cv2.line(
                gray_bgr,
                (x1, y1),
                (x2, y2),
                (255, 200, 200),
                2,
            )

        cv2.imwrite(self.tmp_preview_path, gray_bgr)

        x_edge1 = max(int((center_x - self.cropRange * pitch) / width * width_org), 0)
        y_edge1 = max(int((center_y - self.cropRange * pitch) / height * height_org), 0)
        x_edge2 = min(
            int((center_x + self.cropRange * pitch) / width * width_org), width_org - 1
        )
        y_edge2 = min(
            int((center_y + self.cropRange * pitch) / height * height_org),
            height_org - 1,
        )

        crop_info = [[x_edge1, x_edge2], [y_edge1, y_edge2]]
        log.info(f"crop_info = {crop_info}")

        return crop_info


class QuadPCBBackTrimmer(QuadFrontTrimmer):
    def trim_image(self, params):
        rot_angle = params["rot_angle"]
        init_blur = params["init_blur"]
        contrast = params["contrast"]
        brightness = params["brightness"]
        init_param2 = params["init_param2"]

        image_1 = self.parent.img_bgr.copy()

        image_center = tuple(np.array(image_1.shape[1::-1]) / 2)
        rot_mat = cv2.getRotationMatrix2D(image_center, rot_angle, 1.0)
        image_2 = cv2.warpAffine(
            image_1, rot_mat, image_1.shape[1::-1], flags=cv2.INTER_LINEAR
        )

        image_3 = cv2.convertScaleAbs(image_2, alpha=contrast, beta=brightness)

        height_org, width_org, channel = image_3.shape

        resize_scale = (
            1200 / width_org if width_org >= height_org else 1200 / height_org
        )

        image_4 = cv2.resize(
            image_3, None, None, resize_scale, resize_scale, cv2.INTER_LANCZOS4
        )

        height, width, channel = image_4.shape

        for i in range(height):
            for j in range(width):
                image_3.itemset((i, j, 1), 0)

        # use only Red color
        gray = image_4[:, :, 2]
        cv2.imwrite(self.tmp_preview_path, gray)

        log.info("Searching pickup points...")

        pitch = None
        pickup_points = None

        for blurSize in range(init_blur, 0, -1):
            try:
                pitch, center, pickup_points = self.houghScan(
                    gray, blurSize, init_param2
                )
                break
            except Exception as e:
                log.info(str(e))
                continue

        if not pitch:
            msg = "failed to find any pickup points"
            raise Exception(msg)

        center_x = int(center[0])
        center_y = int(center[1])

        log.info(f"1mm = {pitch:.3f} px, center = ( {center_x:6.3f}, {center_y:6.3f} )")

        x_edge1_0 = max(int(center_x - self.cropRange * pitch), 0)
        y_edge1_0 = max(int(center_y - self.cropRange * pitch), 0)
        x_edge2_0 = min(int(center_x + self.cropRange * pitch), width - 1)
        y_edge2_0 = min(int(center_y + self.cropRange * pitch), height - 1)

        x_edge1 = max(int((center_x - self.cropRange * pitch) / resize_scale), 0)
        y_edge1 = max(int((center_y - self.cropRange * pitch) / resize_scale), 0)
        x_edge2 = min(
            int((center_x + self.cropRange * pitch) / resize_scale), width_org - 1
        )
        y_edge2 = min(
            int((center_y + self.cropRange * pitch) / resize_scale), height_org - 1
        )

        crop_info = [[x_edge1, x_edge2], [y_edge1, y_edge2]]
        log.info(f"crop_info = {crop_info}")

        cropped = image_2[y_edge1:y_edge2, x_edge1:x_edge2].copy()

        cv2.imwrite(self.tmp_cropped_path, cropped)

        marker_width = int(0.005 * width)

        color = (0, 255, 128)

        # Draw the center reticle
        gray_bgr = cv2.cvtColor(gray, cv2.COLOR_GRAY2RGB)

        cv2.line(
            gray_bgr,
            (center_x - 20, center_y),
            (center_x + 20, center_y),
            color,
            marker_width,
        )
        cv2.line(
            gray_bgr,
            (center_x, center_y - 20),
            (center_x, center_y + 20),
            color,
            marker_width,
        )

        cv2.line(
            gray_bgr,
            (x_edge1_0, y_edge1_0),
            (x_edge1_0 + 50, y_edge1_0),
            color,
            marker_width,
        )
        cv2.line(
            gray_bgr,
            (x_edge1_0, y_edge1_0),
            (x_edge1_0, y_edge1_0 + 50),
            color,
            marker_width,
        )

        cv2.line(
            gray_bgr,
            (x_edge2_0, y_edge1_0),
            (x_edge2_0 - 50, y_edge1_0),
            color,
            marker_width,
        )
        cv2.line(
            gray_bgr,
            (x_edge2_0, y_edge1_0),
            (x_edge2_0, y_edge1_0 + 50),
            color,
            marker_width,
        )

        cv2.line(
            gray_bgr,
            (x_edge1_0, y_edge2_0),
            (x_edge1_0 + 50, y_edge2_0),
            color,
            marker_width,
        )
        cv2.line(
            gray_bgr,
            (x_edge1_0, y_edge2_0),
            (x_edge1_0, y_edge2_0 - 50),
            color,
            marker_width,
        )

        cv2.line(
            gray_bgr,
            (x_edge2_0, y_edge2_0),
            (x_edge2_0 - 50, y_edge2_0),
            color,
            marker_width,
        )
        cv2.line(
            gray_bgr,
            (x_edge2_0, y_edge2_0),
            (x_edge2_0, y_edge2_0 - 50),
            color,
            marker_width,
        )

        for p in pickup_points:
            cv2.circle(
                gray_bgr,
                (int(p[0]), int(p[1])),
                int(p[2]),
                color,
                int(marker_width * 0.5),
            )

        cv2.imwrite(self.tmp_preview_path, gray_bgr)

        return crop_info

    def houghScan(self, image_inverted, blurSize, param2):
        log.info(f"  Hough search: blurSize = {blurSize}")

        blur = cv2.blur(image_inverted, (blurSize, blurSize))

        height, width = image_inverted.shape

        deg = math.atan(1) / 45.0

        while True:
            if param2 > 80 or param2 < 10:
                msg = "failure in detection"
                raise Exception(msg)

            _ret, image_4 = cv2.threshold(
                blur, thresh=120, maxval=255, type=cv2.THRESH_TOZERO
            )

            # detect circle
            circles = cv2.HoughCircles(
                image=image_4,
                method=cv2.HOUGH_GRADIENT,
                dp=1.0,
                minDist=min(width, height) * 0.12,
                param1=200,
                param2=param2,
                minRadius=int(min(width, height) * 0.005),
                maxRadius=int(min(width, height) * 0.04),
            )

            pickup_cands = []

            for i in circles[0, :]:
                # check if center is in middle of picture
                if (
                    i[0] < width * 0.05
                    or i[0] > width * 0.95
                    or i[1] < height * 0.05
                    or i[1] > height * 0.95
                ):
                    continue

                # draw the outer circle
                cv2.circle(image_4, (int(i[0]), int(i[1])), int(i[2]), (0, 0, 0), 2)

                # draw the center of the circle
                cv2.circle(image_4, (int(i[0]), int(i[1])), 2, (0, 0, 0), 3)

                pickup_cands += [i]

            log.info(f"param2 = {param2}: #pickup_cands = {len( pickup_cands )}")

            pickup_points = set()

            for i in range(len(pickup_cands)):
                for j in range(i + 1, len(pickup_cands)):
                    for k in range(j + 1, len(pickup_cands)):
                        p1 = pickup_cands[i]
                        p2 = pickup_cands[j]
                        p3 = pickup_cands[k]

                        l3 = [abs(p1[0] - p2[0]), abs(p1[1] - p2[1]), p1, p2, p3]
                        l1 = [abs(p2[0] - p3[0]), abs(p2[1] - p3[1]), p2, p3, p1]
                        l2 = [abs(p3[0] - p1[0]), abs(p3[1] - p1[1]), p3, p1, p2]

                        triangle = [
                            {
                                "v1": [line[2][0], line[2][1]],
                                "v2": [line[3][0], line[3][1]],
                                "v3": [line[4][0], line[4][1]],
                                "dx": line[0],
                                "dy": line[1],
                                "length": math.hypot(line[0], line[1]),
                                "angle": math.atan(line[1] / (line[0] + 1.0e-7)),
                            }
                            for line in [l1, l2, l3]
                        ]

                        horLines = [
                            edge for edge in triangle if edge.get("angle") < 2 * deg
                        ]

                        if len(horLines) != 1:
                            continue

                        horIndex = triangle.index(horLines[0])

                        horLine = triangle[horIndex]

                        # log.info( pprint.pformat( horLine ) )

                        # the horizontal line should come at the bottom
                        if horLine.get("v3")[1] < horLine.get("v1")[1]:
                            continue

                        horLength = triangle[horIndex].get("length")

                        others_ratio = [
                            edge.get("length") / horLength
                            for edge in triangle
                            if edge != horLines[0]
                        ]

                        if not all(
                            0.95 * math.sqrt(5.0) / 2.0
                            < ratio
                            < 1.05 * math.sqrt(5.0) / 2.0
                            for ratio in others_ratio
                        ):
                            continue

                        pickup_points.add((p1[0], p1[1], p1[2]))
                        pickup_points.add((p2[0], p2[1], p2[2]))
                        pickup_points.add((p3[0], p3[1], p3[2]))

                        log.info("")
                        log.info("------------------")
                        for p in pickup_points:
                            log.info(
                                f"    Pickup point: center: ({p[0]:.2f}, {p[1]:.2f}), radius: {p[2]:.2f}"
                            )

                        horY = statistics.mean(
                            [horLine.get("v1")[1], horLine.get("v2")[1]]
                        )
                        centerY = statistics.mean([horY, horLine.get("v3")[1]])
                        centerX = statistics.mean(
                            [horLine.get("v1")[0], horLine.get("v2")[0]]
                        )
                        center = [centerX, centerY]
                        gauge_std = horLength / 54.0  # [ pixels/mm ]

                        return gauge_std, center, pickup_points

            if param2 > 80 or param2 < 10:
                msg = "failure in detection"
                raise Exception(msg)

            param2 -= 1
            # end if while loop

        return None



from __future__ import annotations

import logging

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import (
    QButtonGroup,
    QGroupBox,
    QHBoxLayout,
    QLabel,
    QMessageBox,
    QPushButton,
    QRadioButton,
    QVBoxLayout,
    QWidget,
)

log = logging.getLogger(__name__)


class SelectmodeWindow(QWidget):
    def __init__(self, parent=None):
        super(QWidget, self).__init__()
        self.parent = parent

        layout = QVBoxLayout()
        mode_vbox = QVBoxLayout()
        mode_box = QVBoxLayout()
        button_box = QHBoxLayout()

        label_header = QLabel()
        label_header.setText(
            f'<center><font size="4">module-qc-nonelec-gui (version {self.parent.version})</font></center>'
        )

        label_text = QLabel()
        label_text.setText('<center><font size="6">Select the service</font></center>')

        Next_button = QPushButton("&Next")
        Next_button.clicked.connect(self.next_page)

        button_box.addStretch()
        button_box.addWidget(Next_button)

        radio_QC = self.make_radiobutton("Upload result of QC test to localDB")
        radio_QC.setChecked(True)
        if self.parent.isPractice:
            radio_bare = self.make_radiobutton(
                "Register Bare-module (inavailable at practice mode)"
            )
            radio_bare.setStyleSheet(
                "QRadioButton{font: 12pt; color:gray} QRadioButton::indicator { width: 12px; height: 12px; color: gray;};"
            )
            radio_bare.setCheckable(False)
        else:
            radio_bare = self.make_radiobutton("Register Bare-module")
        self.mode_radio_group = QButtonGroup()
        self.mode_radio_group.addButton(radio_QC, 0)
        self.mode_radio_group.addButton(radio_bare, 1)

        mode_vbox.addWidget(radio_QC)
        mode_vbox.addWidget(radio_bare)
        mode_box.addStretch()
        mode_box.addLayout(mode_vbox)
        mode_box.addStretch()

        mode_group = QGroupBox("")
        mode_group.setStyleSheet("QGroupBox { font-size: 15px;font-weight: bold;} ")
        mode_group.setLayout(mode_box)

        layout.addWidget(label_header)
        layout.addWidget(label_text)
        layout.addStretch()
        layout.addWidget(mode_group)
        layout.addStretch()
        layout.addLayout(button_box)

        self.setLayout(layout)

    def make_radiobutton(self, label):
        radiobutton = QRadioButton(label)
        radiobutton.setCheckable(True)
        radiobutton.setFocusPolicy(Qt.NoFocus)
        radiobutton.setStyleSheet(
            "QRadioButton{font: 12pt;} QRadioButton::indicator { width: 12px; height: 12px;};"
        )
        return radiobutton

    def next_page(self):
        try:
            log.info("[Mode] " + self.mode_radio_group.checkedButton().text())
        except Exception:
            QMessageBox.warning(
                None, "Warning", "Please check each radiobutton.", QMessageBox.Ok
            )

        self.parent.receive_mode(self.mode_radio_group.checkedId())



from __future__ import annotations

import cv2
from PyQt5.QtGui import QImage, QPixmap
from PyQt5.QtWidgets import (
    QGraphicsPixmapItem,
    QGraphicsScene,
    QGraphicsView,
    QGridLayout,
    QPushButton,
    QWidget,
)

from module_qc_nonelec_gui.qc_tests.VISUAL_INSPECTION.functions.cv2_func import (
    img_cvt_rgb,
    read_img,
    write_img,
)


class SplitImageWindow(QWidget):
    def __init__(self, parent=None):
        super(QWidget, self).__init__(parent)
        self.parent = parent

        layout = QGridLayout()

        self.view = QGraphicsView()
        self.scene = QGraphicsScene()
        self.srect = self.view.rect()
        self.view.setScene(self.scene)

        self.parent.update_img(self.parent.img_bgr)
        self.img_bgr_line, h, w, _d = read_img(
            f"{self.parent.temp_dir_path}/img_{self.parent.mode}_{self.parent.rev}.jpg"
        )

        nsplit = self.parent.config.get("nsplit", 6)
        grid_size_w = int(w / nsplit)
        grid_size_h = int(h / nsplit)

        for i in range(nsplit - 1):
            # draw vertical line
            self.img_bgr_line = cv2.line(
                self.img_bgr_line,
                (grid_size_w * (i + 1), 0),
                (grid_size_w * (i + 1), h),
                (0, 255, 128),
                10,
            )
            # draw horizontal line
            self.img_bgr_line = cv2.line(
                self.img_bgr_line,
                (0, grid_size_h * (i + 1)),
                (w, grid_size_h * (i + 1)),
                (0, 255, 128),
                10,
            )

        path_line = f"{self.parent.temp_dir_path}/split_{self.parent.mode}_img.jpg"
        write_img(self.img_bgr_line, path_line)
        img_bgr_line, h_line, w_line, _d_line = read_img(path_line)
        scale = min(600.0 / float(h_line), 600.0 / float(w_line))
        resize_img = cv2.resize(
            img_bgr_line,
            dsize=None,
            fx=scale,
            fy=scale,
            interpolation=cv2.INTER_LANCZOS4,
        )
        img_rgb, h_rgb, w_rgb, d_rgb = img_cvt_rgb(resize_img)
        bytesPerLine = d_rgb * w_rgb

        # # for test
        # self.parent.update_img(img_bgr_line)

        self.image = QImage(
            img_rgb.data, w_rgb, h_rgb, bytesPerLine, QImage.Format_RGB888
        )
        self.item = QGraphicsPixmapItem(QPixmap.fromImage(self.image))
        self.scene.addItem(self.item)
        self.view.setScene(self.scene)
        self.view.setFixedWidth(600)
        self.view.setFixedHeight(600)
        layout.addWidget(self.view, 0, 0)

        button_start = QPushButton("Start Inspection")
        button_start.clicked.connect(self.start)

        layout.addWidget(button_start, 1, 0)
        self.setLayout(layout)

    def start(self):
        self.close()
        self.parent.inspection()



from __future__ import annotations

import csv
import json
import logging
import traceback
from pathlib import Path

import jsonschema
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import (
    QFileDialog,
    QGridLayout,
    QHBoxLayout,
    QLabel,
    QLineEdit,
    QMessageBox,
    QPushButton,
    QScrollArea,
    QTextEdit,
    QVBoxLayout,
    QWidget,
)

log = logging.getLogger(__name__)

# import tkinter


class InitialWindow(QWidget):
    def __init__(self, parent=None):
        super(QWidget, self).__init__(parent)
        self.parent = parent

        ###############################################################
        ########         json schema file path                 ########
        try:
            self.jsonschema_path = Path(__file__).parent.joinpath(
                "lib", "jsonschema", "schema.json"
            )
        except Exception:
            self.jsonschema_path = ""
        ###############################################################

        layout = QVBoxLayout()
        bottom_box = QHBoxLayout()
        grid_layout = QGridLayout()
        # radio_box = QVBoxLayout()

        HBox1 = QHBoxLayout()
        HBox2 = QHBoxLayout()

        label_title = QLabel()
        label_title.setText('<center><font size="7">Metrology</font></center>')

        File_button = QPushButton("&Choose file")
        File_button.clicked.connect(self.choose_file)

        label_result = QLabel()
        label_result.setText("Result file: ")
        label_comment = QLabel()
        label_comment.setText("Comment : ")

        self.edit_comment = QTextEdit()
        self.edit_result = QLineEdit()
        self.edit_result.setFocusPolicy(Qt.NoFocus)
        self.edit_result.setReadOnly(True)

        HBox1.addWidget(self.edit_result)
        HBox1.addWidget(File_button)

        grid_layout.addWidget(label_result, 0, 0)
        grid_layout.addWidget(self.edit_result, 0, 1)
        grid_layout.addWidget(File_button, 0, 2)

        grid_layout.addWidget(label_comment, 1, 0)

        Next_button = QPushButton("&Next")
        Next_button.clicked.connect(self.receive_path)
        Back_button = QPushButton("&Back")
        Back_button.clicked.connect(self.back_page)

        bottom_box.addWidget(Back_button)
        bottom_box.addStretch()
        bottom_box.addWidget(Next_button)

        outer = QScrollArea()
        outer.setWidgetResizable(True)
        outer.setWidget(self.edit_comment)
        HBox2.addWidget(outer)
        grid_layout.addWidget(outer, 1, 1)

        layout.addWidget(label_title)
        layout.addStretch()
        layout.addLayout(grid_layout)
        layout.addLayout(bottom_box)

        self.setLayout(layout)

    def choose_file(self):
        dlg = QFileDialog()
        dlg.setFileMode(QFileDialog.AnyFile)

        try:
            dlg.exec_()
            filename = dlg.selectedFiles()[0]
            self.edit_result.setText(filename)
        except Exception:
            log.exception(traceback.format_exc())
            QMessageBox.warning(
                None, "Error", "Please choose correct file", QMessageBox.Ok
            )

    def receive_path(
        self,
    ):
        try:
            result_doc = None

            with Path(self.edit_result.text()).open(encoding="utf-8") as f:
                if ".csv" in self.edit_result.text():
                    csv_contents = list(csv.reader(f))

                    tmp_dict = {}

                    for i, _col in enumerate(csv_contents[0]):
                        tmp_dict.update({csv_contents[0][i]: csv_contents[1][i]})

                    result_doc = {}

                    result_params = {}

                    for k, v in tmp_dict.items():
                        if "result" in k and "_key" in k:
                            for k2, v2 in tmp_dict.items():
                                if k.replace("_key", "_value") in k2:
                                    result_params.update({v: typed(v2)})
                                    break
                        elif "_value" in k:
                            continue
                        else:
                            result_doc.update({k.replace(" ", ""): typed(v)})

                    result_doc["results"] = result_params

                elif ".json" in self.edit_result.text():
                    result_doc = json.load(f)
                    with Path(self.jsonschema_path).open(encoding="utf-8") as s:
                        jsonschema_dict = json.load(s)
                        jsonschema.validate(result_doc, jsonschema_dict)

            if result_doc["component"] == "EXAMPLE":
                result_doc["component"] = self.parent.parent.testRun.get("serialNumber")
                log.info("module name is EXAMPLE")
                valid_message = QMessageBox.warning(
                    None,
                    "warning",
                    "You use example file!",
                    QMessageBox.Ok | QMessageBox.No,
                )
                if valid_message == QMessageBox.No:
                    return

            if self.parent.parent.testRun.get("serialNumber") != result_doc.get(
                "component"
            ):
                QMessageBox.warning(
                    None,
                    "Error",
                    "Specified serial number in the input file does not match!",
                    QMessageBox.Ok,
                )

                return

        except jsonschema.exceptions.ValidationError:
            log.exception(traceback.format_exc())
            QMessageBox.warning(
                None,
                "Error",
                "JSON File format not correct. \nPlease check the message on CUI.",
                QMessageBox.Ok,
            )
            self.choose_file()
        except Exception:
            log.exception(traceback.format_exc())
            QMessageBox.warning(None, "Error", "Unexpected Error", QMessageBox.Ok)

        free_comment = self.edit_comment.toPlainText()

        try:
            result_doc["comment"] = free_comment
        except Exception:
            QMessageBox.warning(
                None, "Error", "Input file not specified", QMessageBox.Ok
            )

        self.parent.receive_result(result_doc)

    def back_page(self):
        self.parent.close_and_return()


def typed(value):
    try:
        return int(value)
    except Exception:
        try:
            return float(value)
        except Exception:
            if "," in value:
                try:
                    list_str = value.replace("{", "").replace("}", "").split(",")

                    try:
                        return [int(v) for v in list_str]
                    except Exception:
                        try:
                            return [float(v) for v in list_str]
                        except Exception:
                            return list_str

                except Exception:
                    return value

            elif value == "TRUE":
                return True
            elif value == "FALSE":
                return False
            else:
                return value



from __future__ import annotations

import csv
import logging
from pathlib import Path

from PyQt5.QtCore import Qt
from PyQt5.QtGui import QDoubleValidator
from PyQt5.QtWidgets import (
    QComboBox,
    QFileDialog,
    QHBoxLayout,
    QLabel,
    QLineEdit,
    QMessageBox,
    QPushButton,
    QScrollArea,
    QTextEdit,
    QVBoxLayout,
    QWidget,
)

log = logging.getLogger(__name__)


class InitialWindow(QWidget):
    def __init__(self, parent=None):
        super(QWidget, self).__init__(parent)
        self.parent = parent

        layout = QVBoxLayout()
        bottom_box = QHBoxLayout()
        comment_layout = QVBoxLayout()

        label_title = QLabel()
        label_title.setText(
            '<center><font size="5">Input the result of wirebond pull test</font></center>'
        )

        Next_button = QPushButton("&Next")
        Next_button.clicked.connect(self.pass_result)
        Back_button = QPushButton("&Back")
        Back_button.clicked.connect(self.back_page)

        bottom_box.addWidget(Back_button)
        bottom_box.addStretch()
        bottom_box.addWidget(Next_button)

        self.pull_strength_fields = []
        self.break_mode_fields = []
        self.properties = []

        layout_properties = QVBoxLayout()
        layout_properties.addLayout(self.add_text_input("Operator", "OPERATOR"))
        layout_properties.addLayout(self.add_text_input("Instrument", "INSTRUMENT"))

        inner = QScrollArea()
        inner.setFixedWidth(600)
        inner.setFixedHeight(400)
        result_wid = QWidget()
        result_layout = QVBoxLayout()
        for i in range(32):
            result_layout.addLayout(self.create_pull_slot(i))

        result_wid.setLayout(result_layout)
        inner.setWidget(result_wid)

        select_button = QPushButton("Select CSV...")
        select_button.clicked.connect(self.select_file)

        label_comment = QLabel()
        label_comment.setText("Comment : ")

        self.edit_result = QLineEdit()
        self.edit_result.setFocusPolicy(Qt.NoFocus)
        self.edit_result.setReadOnly(True)
        self.edit_result.setFixedWidth(500)

        label_manual = QLabel()
        label_manual.setText("Additional Manual Inputs:")

        self.edit_comment = QTextEdit()
        self.edit_comment.setText(self.parent.result_info["comment"])

        comment_layout.addWidget(label_comment)
        comment_layout.addWidget(self.edit_comment)

        csv_layout = QHBoxLayout()
        csv_layout.addWidget(select_button)
        csv_layout.addWidget(self.edit_result)

        layout.addWidget(label_title)
        layout.addStretch()
        layout.addLayout(layout_properties)
        layout.addLayout(csv_layout)
        layout.addWidget(label_manual)
        layout.addWidget(inner)
        layout.addWidget(label_comment)
        layout.addLayout(comment_layout)
        layout.addLayout(bottom_box)

        self.setLayout(layout)

    def add_text_input(self, title, code):
        layout = QHBoxLayout()
        label = QLabel()
        label.setText(title + ": ")

        property_input = QLineEdit()
        property_input.setFixedWidth(200)

        self.properties += [{"code": code, "field": property_input}]

        layout.addWidget(label)
        layout.addWidget(property_input)
        layout.addStretch()

        return layout

    def create_pull_slot(self, index):
        layout = QHBoxLayout()
        label = QLabel()
        label.setText(f"<b>Wire #{index+1}</b>      ")

        label_strength = QLabel()
        label_strength.setText("Pull Strength [g]: ")

        strength_field = QLineEdit()
        strength_field.setAlignment(Qt.AlignRight)
        strength_field.setValidator(QDoubleValidator(0.0, 20.0, 2))
        strength_field.setFixedWidth(50)
        strength_field.setMaxLength(5)

        layout.addWidget(label)
        layout.addWidget(label_strength)
        layout.addWidget(strength_field)

        self.break_modes = [
            "Heel break on chip",
            "Heel break on hybrid",
            "Midspan break",
            "Bond peel on chip",
            "Bond peel on hybrid",
            "Operator error",
            "other error",
        ]

        combo = QComboBox()
        for mode in self.break_modes:
            combo.addItem(mode)
        combo.setPlaceholderText("Select the breaking mode...")
        combo.setCurrentIndex(-1)

        self.pull_strength_fields += [strength_field]
        self.break_mode_fields += [combo]

        layout.addWidget(combo)

        return layout

    def pass_result(self):
        for prop in self.properties:
            if prop["field"].text() == "":
                QMessageBox.warning(
                    self,
                    "Warning",
                    f"Empty {prop['code']} value!",
                    QMessageBox.Ok,
                )
                return

            self.parent.parent.testRun["results"]["property"][prop["code"]] = prop[
                "field"
            ].text()

        sample_file = self.edit_result.text()

        pull_data = []

        try:
            with Path(sample_file).open(encoding="utf-8") as f:
                reader = csv.reader(f)

                for index, row in enumerate(reader):
                    # validation
                    if len(row) < 2:
                        QMessageBox.warning(
                            self,
                            "Warning",
                            f"Wire #{index+1} info is complete (pull strength and breaking mode need to be specified):\n {row}",
                            QMessageBox.Ok,
                        )
                        return

                    try:
                        strength = float(row[0])

                    except Exception:
                        QMessageBox.warning(
                            self,
                            "Warning",
                            f"Wire #{index+1} strength needs to be a float number (unit: [g])",
                            QMessageBox.Ok,
                        )
                        return

                    if strength <= 0.0:
                        QMessageBox.warning(
                            self,
                            "Warning",
                            f"Wire #{index+1} strength needs to be a positive value",
                            QMessageBox.Ok,
                        )
                        return

                    if row[1] not in self.break_modes:
                        QMessageBox.warning(
                            self,
                            "Warning",
                            f"Wire #{index+1}: breaking mode needs to be one of {self.breaking_modes}",
                            QMessageBox.Ok,
                        )
                        return

                # Validation is done, filling

            with Path(sample_file).open(encoding="utf-8") as f:
                reader = csv.reader(f)

                for row in reader:
                    pull_data += [{"strength": float(row[0]), "break_mode": row[1]}]

        except Exception:
            pass

        for index, fields in enumerate(
            zip(self.pull_strength_fields, self.break_mode_fields)
        ):
            strength = fields[0]
            mode = fields[1]
            try:
                pull_data += [
                    {
                        "strength": float(strength.text()),
                        "break_mode": mode.currentText(),
                    }
                ]
            except Exception:
                if strength.text() != "":
                    QMessageBox.warning(
                        self,
                        "Warning",
                        f"Wire #{index+1} has an input error",
                        QMessageBox.Ok,
                    )
                    return

        if len(pull_data) < 8:
            QMessageBox.warning(
                self,
                "Warning",
                f"At least 8 measurements are required (current input: {len(pull_data)})",
                QMessageBox.Ok,
            )
            return

        self.parent.parent.testRun["results"]["metadata"]["pull_data"] = pull_data

        self.parent.parent.testRun["results"][
            "comment"
        ] = self.edit_comment.toPlainText()

        self.parent.parent.receive_result(self)

    def select_file(self):
        dlg = QFileDialog()
        dlg.setFileMode(QFileDialog.AnyFile)

        try:
            dlg.exec_()
            filename = dlg.selectedFiles()[0]
            self.edit_result.setText(filename)
        except Exception:
            QMessageBox.warning(
                None, "Warning", "Please select a correct file", QMessageBox.Ok
            )

    def back_page(self):
        self.parent.close_and_return()



from __future__ import annotations

import cv2
import numpy as np

from module_qc_nonelec_gui.qc_tests.vi.functions.cv2_func import cal_distance, math


def detect_circle(img):
    h, w, c = img.shape
    # remove blue pixel
    for i in range(h):
        for j in range(w):
            img.itemset((i, j, 0), 0)

    # get gray image
    gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
    gray = cv2.medianBlur(gray, 5)

    # Circle detection with Hough Transformation
    circles = cv2.HoughCircles(
        gray,
        cv2.HOUGH_GRADIENT,
        1,
        100,
        param1=100,
        param2=50,
        minRadius=50,
        maxRadius=70,
    )
    circles = np.uint16(np.around(circles))

    # sort with y col
    col_num = 1
    circles = circles[0, :]
    return circles[np.argsort(circles[:, col_num])]


def find_edge(img, circles, mini_ref_points1, mini_ref_points2, config):
    h, w, c = img.shape

    # get interesting circles
    x = circles[:, 0]
    x_hist, x_bins = np.histogram(x, bins=20)
    ref_line_x = []
    for i, x_value in enumerate(x_hist):
        if x_value > 6:
            ref_line_x.append([x_bins[i], x_bins[i + 1]])

    # get reference points
    points = {}
    for i, p in enumerate(ref_line_x):
        n = 0
        for circle in circles:
            if p[0] < circle[0] and circle[0] < p[1]:
                n = n + 1
                if n == 1:
                    points[i] = circle.reshape(1, -1)
                else:
                    points[i] = np.vstack([points[i], circle.reshape(1, -1)])

    # calculate rotation angle
    nmatch = 0
    dx = 0
    dy = 0
    matched_points = {}
    distance = 100
    for circle1 in points[0]:
        for circle2 in points[1]:
            if abs(int(circle1[1]) - int(circle2[1])) < distance:
                nmatch = nmatch + 1
                dx = dx - int(circle1[0]) + int(circle2[0])
                dy = dy - int(circle1[1]) + int(circle2[1])
                if nmatch == 1:
                    matched_points[0] = circle1.reshape(1, -1)
                    matched_points[1] = circle2.reshape(1, -1)
                else:
                    matched_points[0] = np.vstack(
                        [matched_points[0], circle1.reshape(1, -1)]
                    )
                    matched_points[1] = np.vstack(
                        [matched_points[1], circle2.reshape(1, -1)]
                    )
    missing_point = -1
    if nmatch == 5:
        if points[0].shape[0] > 6:
            for i, circle1 in enumerate(points[0]):
                for circle2 in points[1]:
                    if abs(int(circle1[1]) - int(circle2[1])) < distance:
                        pass
                    else:
                        if i not in [3, 4]:
                            missing_point = i
        elif points[1].shape[0] > 6:
            for i, circle1 in enumerate(points[1]):
                for circle2 in points[0]:
                    if abs(int(circle1[1]) - int(circle2[1])) < distance:
                        pass
                    else:
                        if i not in [3, 4]:
                            missing_point = i

    dx = dx / nmatch
    dy = dy / nmatch
    theta = math.atan2(dy, dx)
    degree = math.degrees(theta)

    # rotate image
    mat = cv2.getRotationMatrix2D((w / 2, h / 2), degree, 1)
    # affine_img = cv2.warpAffine(img, mat, (w, h))

    for i in range(nmatch):
        a0 = np.vstack([matched_points[0][i, :2].reshape(2, 1), 1])
        a1 = np.vstack([matched_points[1][i, :2].reshape(2, 1), 1])
        a0 = np.dot(mat, a0)
        a1 = np.dot(mat, a1)

        matched_points[0][i, :2] = a0[:2,].reshape(1, 2)
        matched_points[1][i, :2] = a1[:2,].reshape(1, 2)

    if nmatch == 6:
        p00_x = int(matched_points[0][0, 0])
        p00_y = int(matched_points[0][0, 1])
        p01_x = int(matched_points[1][0, 0])
        p01_y = int(matched_points[1][0, 1])
        p50_x = int(matched_points[0][5, 0])
        p50_y = int(matched_points[0][5, 1])
        p51_x = int(matched_points[1][5, 0])
        p51_y = int(matched_points[1][5, 1])

        ref_p00_x = int(mini_ref_points1[0, 0])
        ref_p00_y = int(mini_ref_points1[0, 1])
        ref_p01_x = int(mini_ref_points2[0, 0])
        ref_p01_y = int(mini_ref_points2[0, 1])
        ref_p50_x = int(mini_ref_points1[5, 0])
        ref_p50_y = int(mini_ref_points1[5, 1])
        ref_p51_x = int(mini_ref_points2[5, 0])
        ref_p51_y = int(mini_ref_points2[5, 1])

    elif nmatch == 5:
        if missing_point == 0:
            p00_x = int(matched_points[0][0, 0])
            p00_y = int(matched_points[0][0, 1])
            p01_x = int(matched_points[1][0, 0])
            p01_y = int(matched_points[1][0, 1])
            p50_x = int(matched_points[0][4, 0])
            p50_y = int(matched_points[0][4, 1])
            p51_x = int(matched_points[1][4, 0])
            p51_y = int(matched_points[1][4, 1])

            ref_p00_x = int(mini_ref_points1[1, 0])
            ref_p00_y = int(mini_ref_points1[1, 1])
            ref_p01_x = int(mini_ref_points2[1, 0])
            ref_p01_y = int(mini_ref_points2[1, 1])
            ref_p50_x = int(mini_ref_points1[5, 0])
            ref_p50_y = int(mini_ref_points1[5, 1])
            ref_p51_x = int(mini_ref_points2[5, 0])
            ref_p51_y = int(mini_ref_points2[5, 1])
        elif missing_point in [6, 7]:
            p00_x = int(matched_points[0][0, 0])
            p00_y = int(matched_points[0][0, 1])
            p01_x = int(matched_points[1][0, 0])
            p01_y = int(matched_points[1][0, 1])
            p50_x = int(matched_points[0][4, 0])
            p50_y = int(matched_points[0][4, 1])
            p51_x = int(matched_points[1][4, 0])
            p51_y = int(matched_points[1][4, 1])

            ref_p00_x = int(mini_ref_points1[0, 0])
            ref_p00_y = int(mini_ref_points1[0, 1])
            ref_p01_x = int(mini_ref_points2[0, 0])
            ref_p01_y = int(mini_ref_points2[0, 1])
            ref_p50_x = int(mini_ref_points1[4, 0])
            ref_p50_y = int(mini_ref_points1[4, 1])
            ref_p51_x = int(mini_ref_points2[4, 0])
            ref_p51_y = int(mini_ref_points2[4, 1])
        else:
            p00_x = int(matched_points[0][0, 0])
            p00_y = int(matched_points[0][0, 1])
            p01_x = int(matched_points[1][0, 0])
            p01_y = int(matched_points[1][0, 1])
            p50_x = int(matched_points[0][4, 0])
            p50_y = int(matched_points[0][4, 1])
            p51_x = int(matched_points[1][4, 0])
            p51_y = int(matched_points[1][4, 1])

            ref_p00_x = int(mini_ref_points1[0, 0])
            ref_p00_y = int(mini_ref_points1[0, 1])
            ref_p01_x = int(mini_ref_points2[0, 0])
            ref_p01_y = int(mini_ref_points2[0, 1])
            ref_p50_x = int(mini_ref_points1[5, 0])
            ref_p50_y = int(mini_ref_points1[5, 1])
            ref_p51_x = int(mini_ref_points2[5, 0])
            ref_p51_y = int(mini_ref_points2[5, 1])

    d1 = cal_distance(p00_x, p00_y, p50_x, p50_y)
    d2 = cal_distance(p00_x, p00_y, p01_x, p01_y)
    d3 = cal_distance(p00_x, p00_y, p51_x, p51_y)

    d1_ref = cal_distance(ref_p00_x, ref_p00_y, ref_p50_x, ref_p50_y)
    d2_ref = cal_distance(ref_p00_x, ref_p00_y, ref_p01_x, ref_p01_y)
    d3_ref = cal_distance(ref_p00_x, ref_p00_y, ref_p51_x, ref_p51_y)

    scale = (d1 / d1_ref + d2 / d2_ref + d3 / d3_ref) / 3

    xs = (int(p00_x) + int(p50_x)) / 2 - scale * (int(ref_p00_x) + int(ref_p50_x)) / 2
    xe = (int(p01_x) + int(p51_x)) / 2 + scale * (
        config["width"] - (int(ref_p01_x) + int(ref_p51_x)) / 2
    )
    xs = max(xs, 0)
    xe = min(xe, w)

    ys = (int(p00_y) + int(p01_y)) / 2 - scale * (int(ref_p00_y) + int(ref_p01_y)) / 2
    ye = (int(p50_y) + int(p51_y)) / 2 + scale * (
        config["height"] - (int(ref_p50_y) + int(ref_p51_y)) / 2
    )
    ys = max(ys, 0)
    ye = min(ye, h)

    # img_trim = img_affine[int(ys):int(ye),int(xs):int(xe)]

    # return img_trim
    return dx, dy, ys, ye, xs, xe



from __future__ import annotations

import contextlib
import logging
import re
from functools import reduce

import cv2
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QDoubleValidator, QImage, QPixmap, qt_set_sequence_auto_mnemonic
from PyQt5.QtWidgets import (
    QCheckBox,
    QGraphicsPixmapItem,
    QGraphicsScene,
    QGraphicsView,
    QGridLayout,
    QHBoxLayout,
    QLabel,
    QLineEdit,
    QMessageBox,
    QPlainTextEdit,
    QPushButton,
    QScrollArea,
    QSizePolicy,
    QTabWidget,
    QVBoxLayout,
    QWidget,
)

# import checklist
# import checklist_Module
# import checklist_PCB
from module_qc_nonelec_gui.qc_tests.VISUAL_INSPECTION.functions.cv2_func import (
    img_cvt_rgb,
    read_img,
    write_img,
)

log = logging.getLogger(__name__)


class InspectionWindow(QWidget):
    def __init__(self, parent=None):
        super(QWidget, self).__init__(parent)
        self.parent = parent
        self.brightness = None
        self.contrast = None

        self.TabWidget = None

        layout = QHBoxLayout()

        switch_button_layout = QHBoxLayout()

        self.img_bgr = self.parent.img_bgr
        self.img_h = self.parent.img_h
        self.img_w = self.parent.img_w

        ## prepare target images
        self.path_target = self.split_img(self.img_bgr, self.parent.n_page)
        self.path_gm = self.parent.path_gm + f"/tile{self.parent.n_page}.jpg"

        self.view = self.setImg(self.path_target, 0)
        self.view_gm = self.setImg(self.path_gm, 1)

        label_page = QLabel(self)
        label_page.setText(f"page: {self.parent.n_page + 1}/{self.parent.tot_page}")

        label_message = QLabel(self)
        label_message.setText("Tick checkboxes for identified defects.")

        label_message2 = QLabel(self)

        label_message2.setText(
            self.parent.msg_map.get(self.parent.parent.info_dict.get("currentStage"))
        )

        label_comment = QLabel(self)
        label_comment.setText("Describe observation and details of defects:")

        self.edit_comment = QPlainTextEdit(self)
        self.edit_comment.setFixedHeight(100)
        with contextlib.suppress(Exception):
            self.edit_comment.setText(self.parent.comment_dic[str(self.parent.n_page)])

        ## prepare buttons
        label_brightness = QLabel(self)
        label_brightness.setText("Brightness: ")

        label_contrast = QLabel(self)
        label_contrast.setText("Contrast: ")

        self.input_brightness = QLineEdit(self)
        self.input_brightness.setFixedWidth(30)
        self.input_brightness.setValidator(QDoubleValidator(-100, 100, 0))
        self.input_brightness.setText(str(15))

        self.input_contrast = QLineEdit(self)
        self.input_contrast.setFixedWidth(30)
        self.input_contrast.setValidator(QDoubleValidator(0.0, 2.0, 1))
        self.input_contrast.setText(str(1.3))

        switch_button_layout.addWidget(label_brightness)
        switch_button_layout.addWidget(self.input_brightness)
        switch_button_layout.addWidget(label_contrast)
        switch_button_layout.addWidget(self.input_contrast)

        self.input_brightness.returnPressed.connect(self.retouch)
        self.input_contrast.returnPressed.connect(self.retouch)

        self.button_checked = QPushButton("Checkout This Tile", self)
        self.button_checked.setCheckable(True)
        self.button_checked.clicked.connect(self.checkout)
        self.button_checked.setChecked(self.parent.page_checked[self.parent.n_page])

        button_back = QPushButton("Back", self)
        button_back.clicked.connect(self.back_page)
        button_next = QPushButton("Next", self)
        button_next.clicked.connect(self.next_page)

        if self.parent.type_name == "PCB":
            pass
            # button_vanilla = QPushButton("Vanilla", self)
            # button_vanilla.clicked.connect(self.setVanillaImg)
            # button_platting = QPushButton("Platting", self)
            # button_platting.clicked.connect(self.setPlattingImg)
            # button_smd = QPushButton("SMD", self)
            # button_smd.clicked.connect(self.setSMDImg)

        if self.parent.type_name == "MODULE":
            if self.parent.stage == "MODULETOPCB":
                button_vanilla = QPushButton("Vanilla", self)
                button_vanilla.clicked.connect(self.setVanillaImg)
                button_platting = QPushButton("Platting", self)
                button_platting.clicked.connect(self.setPlattingImg)
                button_smd = QPushButton("SMD", self)
                button_smd.clicked.connect(self.setSMDImg)

                switch_button_layout.addWidget(button_vanilla)
                switch_button_layout.addWidget(button_platting)
                switch_button_layout.addWidget(button_smd)

            else:
                button_vanilla = QPushButton("Vanilla", self)
                button_vanilla.clicked.connect(self.setVanillaImg)
                button_wire = QPushButton("Wire", self)
                button_wire.clicked.connect(self.setWireImg)

                switch_button_layout.addWidget(button_vanilla)
                switch_button_layout.addWidget(button_wire)

        self.checklist_gen(self.parent.n_page, self.parent.tot_page)

        cb_layout_list = []
        for first_key, value in self.cb.items():
            cb_layout_list.append(QVBoxLayout())
            cb_layout_list[-1].setAlignment(Qt.AlignTop)
            label = QLabel()
            label.setText(first_key + " Defects")
            label.resize(200, 30)
            cb_layout_list[-1].addWidget(label)
            for _second_key, checkbox in value.items():
                cb_layout_list[-1].addWidget(checkbox)

        cb_layout = QHBoxLayout()
        for i in cb_layout_list:
            cb_layout.addLayout(i)

        cb_widget = QWidget()
        cb_widget.setLayout(cb_layout)

        outer = QScrollArea()
        outer.setWidgetResizable(True)
        outer.setWidget(cb_widget)

        input_layout = QGridLayout()
        input_layout.addWidget(label_message, 0, 0)
        input_layout.addWidget(label_message2, 1, 0)
        input_layout.addWidget(outer, 2, 0)
        input_layout.addWidget(label_comment, 3, 0)
        input_layout.addWidget(self.edit_comment, 4, 0)
        input_layout.addWidget(self.button_checked, 5, 0)

        button_layout = QHBoxLayout()
        button_layout.addWidget(button_back)
        button_layout.addStretch()
        button_layout.addWidget(button_next)

        switch_button_layout.addStretch()
        switch_button_layout.addWidget(label_page)

        control_layout = QVBoxLayout()
        control_layout.addLayout(switch_button_layout)
        control_layout.addLayout(input_layout)
        control_layout.addLayout(button_layout)

        layout.addLayout(self.make_tab_layout())
        #        layout.addLayout( input_layout )
        layout.addLayout(control_layout)
        self.setLayout(layout)
        self.setFixedSize(layout.sizeHint())

    def keyPressEvent(self, event):
        if event.key() == 83:  # s
            self.TabWidget.setCurrentIndex(1 - self.TabWidget.currentIndex())
        if event.key() == 88:  # x
            self.button_checked.setChecked(True)
            self.checkout()
        if event.key() == 70:  # f
            pageStatus = self.parent.page_checked[self.parent.n_page]
            if pageStatus is False:
                QMessageBox.warning(
                    self,
                    "Warning",
                    "The tile is not checked out!",
                )
            else:
                self.next_page()

            return

        if event.key() == 66:  # b
            self.back_page()

    def back_page(self):
        self.anomaly_update()
        if self.parent.n_page == 0:
            QMessageBox.warning(
                self,
                "Warning",
                "This is the first tile",
            )
        else:
            self.parent.n_page = self.parent.n_page - 1

        self.parent.inspection()

    def next_page(self):
        self.anomaly_update()

        if self.check_status():
            log.info("Inspected all tiles ==> goto summary")
            self.close()
            self.parent.go_to_summary(self)

        else:
            if self.parent.n_page == self.parent.tot_page - 1:
                QMessageBox.warning(
                    self,
                    "Warning",
                    "This is the last tile",
                )
            else:
                self.parent.n_page = self.parent.n_page + 1

            self.parent.inspection()

    def setImg(self, path, index, contrast=1.3, brightness=15.0):
        view = QGraphicsView(self)

        log.info(f"setImg(): path = {path}")

        pictRefSize = 800.0
        img, h, w, d = read_img(path)
        scale = min(pictRefSize / float(h), pictRefSize / float(w))
        imgResizedForQtWindow = cv2.resize(
            img, dsize=None, fx=scale, fy=scale, interpolation=cv2.INTER_LANCZOS4
        )
        out = cv2.convertScaleAbs(
            imgResizedForQtWindow, alpha=contrast, beta=brightness
        )
        img_rgb, h, w, d = img_cvt_rgb(out)
        bytesPerLine = d * w
        qImage = QImage(img_rgb.data, w, h, bytesPerLine, QImage.Format_RGB888)
        pixItem = QGraphicsPixmapItem(QPixmap.fromImage(qImage))
        pixItem.setPixmap(pixItem.pixmap())

        scene = QGraphicsScene()
        scene.addItem(pixItem)
        view.setScene(scene)
        rect = scene.sceneRect()
        log.info(
            f"setImg(): scene {rect.x()}, {rect.y()}, {rect.width()}, {rect.height()}"
        )
        # view.setAlignment(Qt.AlignTop | Qt.AlignLeft)
        # view.fitInView( -400, -400, 800, 800, Qt.KeepAspectRatio)

        if self.TabWidget:
            tab = self.TabWidget.widget(index)
            tab.layout().itemAt(0).widget().deleteLater()
            tab.layout().addWidget(view)

        return view

    def retouch(self):
        ## prepare target images
        self.path_target = self.split_img(self.img_bgr, self.parent.n_page)

        self.brightness = float(self.input_brightness.text())
        self.contrast = float(self.input_contrast.text())

        log.info(
            f"retouch(): retouching the target image with brightness = {self.brightness}, contrast = {self.contrast}"
        )

        self.view = self.setImg(
            self.path_target,
            0,
            self.contrast,
            self.brightness,
        )
        self.view_gm = self.setImg(self.path_gm, 1, 1.3, 15.0)

    def setVanillaImg(self):
        self.setImg(self.path_gm, 1)

    def setPlattingImg(self):
        try:
            self.setImg(
                self.parent.path_gm + f"/tile{self.parent.n_page}_plating.jpg", 1
            )
        except Exception:
            self.setImg(self.path_gm, 1)

    def setSMDImg(self):
        try:
            self.setImg(self.parent.path_gm + f"/tile{self.parent.n_page}_SMD.png", 1)
        except Exception:
            self.setImg(self.path_gm, 1)

    def setWireImg(self):
        try:
            self.setImg(
                self.parent.path_gm + f"/tile{self.parent.n_page}_wireline.jpg", 1
            )
        except Exception:
            self.setImg(self.path_gm, 1)

    def checkout(self):
        self.anomaly_update()
        self.parent.comment_dic[
            str(self.parent.n_page)
        ] = self.edit_comment.toPlainText()

        self.parent.page_checked[self.parent.n_page] = True

    def check_status(self):
        n_checked = reduce(lambda a, b: int(a) + int(b), self.parent.page_checked)

        total = len(self.parent.page_checked)

        log.info(f"checked tiles: {n_checked} / {total}")

        return n_checked == total

    def checkBoxChangeAction(self, state, element):
        if Qt.Checked == state:
            try:
                self.parent.anomaly_dic[str(self.parent.n_page)].append(element)
            except Exception:
                self.parent.anomaly_dic[str(self.parent.n_page)] = [element]
            pathexsit = False
            try:
                for path in self.parent.img_dic.values():
                    p = re.split("[/_.]", path)[3]
                    # print("current page is " + p)
                    if str(self.parent.n_page) == p:
                        pathexsit = True
                if not pathexsit:
                    self.parent.img_dic[str(self.parent.n_page)] = self.path_target
            except Exception:
                self.parent.img_dic[str(self.parent.n_page)] = self.path_target
            # print("anomaly at " + element)
        else:
            self.parent.anomaly_dic[str(self.parent.n_page)].remove(element)
            del self.parent.img_dic[str(self.parent.n_page)]

    def anomaly_update(self):
        categories = list(self.parent.checklist_dict[str(self.parent.n_page)].keys())

        for category in categories:
            items = self.parent.checklist_dict[str(self.parent.n_page)][category]
            for index, item in enumerate(items):
                try:
                    state = self.cb[category][index].checkState()
                    self.checkbox_action(category, state, item)
                except Exception:
                    pass
        self.image_update()

    def checkbox_action(self, category, state, item):
        defectId = ".".join(
            [category, item.replace(" ", "_").replace("(", "").replace(")", "")]
        )

        if state == Qt.Checked:
            if str(self.parent.n_page) not in self.parent.anomaly_dic:
                self.parent.anomaly_dic[str(self.parent.n_page)] = [defectId]
            elif defectId not in self.parent.anomaly_dic[str(self.parent.n_page)]:
                self.parent.anomaly_dic[str(self.parent.n_page)].append(defectId)
        else:
            if (
                str(self.parent.n_page) in self.parent.anomaly_dic
                and defectId in self.parent.anomaly_dic[str(self.parent.n_page)]
            ):
                self.parent.anomaly_dic[str(self.parent.n_page)].remove(defectId)

    def image_update(self):
        is_anomaly = False
        if str(self.parent.n_page) in self.parent.anomaly_dic:
            if len(self.parent.anomaly_dic[str(self.parent.n_page)]):
                is_anomaly = True
        elif len(self.edit_comment.toPlainText()) > 0:
            is_anomaly = True

        if is_anomaly:
            if str(self.parent.n_page) not in self.parent.img_dic:
                self.parent.img_dic[
                    str(self.parent.n_page)
                ] = f"{self.parent.temp_dir_path}/tile_img_{self.parent.mode}_{self.parent.n_page}.jpg"
        else:
            if str(self.parent.n_page) in self.parent.img_dic:
                del self.parent.img_dic[str(self.parent.n_page)]

    def write_data(self):
        self.parent.write_to_localdb()

    def split_img(self, _img, n_page):
        col, row = divmod(n_page, self.parent.nsplit)
        ncol, _mod = divmod(self.parent.tot_page, self.parent.nsplit)
        assert _mod == 0
        nrow = n_page / ncol

        grid_size_w = int(self.img_w / self.parent.nsplit)
        grid_size_h = int(self.img_h / self.parent.nsplit)

        y1 = grid_size_w * row
        y2 = grid_size_w * (row + 1)
        x1 = grid_size_h * col
        x2 = grid_size_h * (col + 1)

        if row == 0:
            y1 = 0
        elif row == nrow - 1:
            y1 -= 100
        else:
            y1 -= 50

        if row == 0:
            y2 += 100
        elif row == nrow - 1:
            y2 = self.img_w
        else:
            y2 += 50

        if col == 0:
            x1 = 0
        elif col == ncol - 1:
            x1 -= 100
        else:
            x1 -= 50

        if col == 0:
            x2 += 100
        elif col == ncol - 1:
            x2 = self.img_h
        else:
            x2 += 50

        log.info(
            f"split_img(): {self.parent.n_page} / {self.parent.tot_page}: [{y1}:{y2}, {x1}:{x2}] "
        )
        self.tile_img_bgr = self.img_bgr[x1:x2, y1:y2]

        write_img(
            self.tile_img_bgr,
            f"{self.parent.temp_dir_path}/tile_img_{self.parent.mode}_{self.parent.n_page}.jpg",
        )

        return f"{self.parent.temp_dir_path}/tile_img_{self.parent.mode}_{self.parent.n_page}.jpg"

    def checklist_gen(self, page, _tot_page):
        self.cb = {}

        defectTypes = self.parent.checklist_dict.get(str(page))

        defects = self.parent.anomaly_dic.get(str(page))

        categories = list(defectTypes.keys())

        for _index_key, key in enumerate(categories):
            items = defectTypes[key]
            # nitems = len(items)
            # interval = int(h / (nitems + 1))
            self.cb[key] = {}
            for index, item in enumerate(items):
                self.cb[key][index] = QCheckBox(item, self)
                if key == "Wire":
                    _q, _mod = divmod(index, 20)

                try:
                    if defects is None:
                        continue

                    defectId = ".".join(
                        [key, item.replace(" ", "_").replace("(", "").replace(")", "")]
                    )

                    if defectId in defects:
                        self.cb[key][index].toggle()

                except Exception as e:
                    log.exception(str(e))

    def make_tab_layout(self):
        layout = QHBoxLayout()

        self.TabWidget = QTabWidget()
        self.TabWidget.resize(850, 850)
        self.TabWidget.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)

        for view, title in zip([self.view, self.view_gm], ["&Target", "&Reference"]):
            self.add_tab(
                self.make_tab(view),
                self.TabWidget,
                title,
            )

        layout.addWidget(self.TabWidget)
        return layout

    def add_tab(self, layout, TabWidget, tab_label):
        qt_set_sequence_auto_mnemonic(True)

        tab = QWidget()
        tab.setFixedWidth(850)
        tab.setFixedHeight(850)
        tab.setLayout(layout)

        TabWidget.addTab(tab, tab_label)

    def make_tab(self, view):
        layout = QHBoxLayout()
        layout.addWidget(view)

        return layout



from __future__ import annotations

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import (
    QFormLayout,
    QHBoxLayout,
    QLabel,
    QLineEdit,
    QPushButton,
    QScrollArea,
    QTextEdit,
    QVBoxLayout,
    QWidget,
)


class ConfirmWindow(QWidget):
    def __init__(self, parent=None):
        super(QWidget, self).__init__(parent)
        self.parent = parent

        titlebox = QVBoxLayout()
        layout = QVBoxLayout()
        button_box = QHBoxLayout()
        inner_box = QVBoxLayout()

        label_title = QLabel()
        label_title.setText(
            '<center><font size="5">Confirm before uploading to the database</font></center>'
        )
        label_practice = QLabel()
        label_practice.setText(
            '<center><font size="4" color = "green"> Practice Mode</font></center>'
        )

        Upload_button = QPushButton("&Upload!")
        Upload_button.clicked.connect(self.upload_to_db)
        json_button = QPushButton("&Check json (for expert)")
        json_button.clicked.connect(self.check_json)
        back_button = QPushButton("&Back")
        back_button.clicked.connect(self.back_page)

        titlebox.addWidget(label_title)

        button_box.addWidget(back_button)
        button_box.addStretch()
        button_box.addWidget(json_button)
        button_box.addWidget(Upload_button)

        inner = QScrollArea()
        inner.setFixedWidth(600)
        inner.setFixedHeight(600)
        result_wid = QWidget()
        result_wid.setLayout(self.make_layout())

        inner.setWidgetResizable(True)
        inner.setWidget(result_wid)
        inner_box.addWidget(inner)

        layout.addLayout(titlebox)
        layout.addLayout(inner_box)
        layout.addLayout(button_box)

        self.setLayout(layout)

    def back_page(self):
        self.parent.back_to_test()

    def check_json(self):
        self.parent.confirm_json()

    def upload_to_db(self):
        self.parent.upload_to_db()

    def add_info(self, Form_layout, label_str, form_text):
        label = QLabel()
        label.setText(label_str)

        if label_str.lower().find("comment") > 0:
            inner = QTextEdit()
            inner.setText(form_text)
            inner.setReadOnly(True)
            inner.setFocusPolicy(Qt.NoFocus)
            inner.setStyleSheet("background-color : linen; color: black;")

            editor = QScrollArea()
            editor.setWidgetResizable(True)
            editor.setWidget(inner)

        else:
            editor = QLineEdit()
            editor.setFixedWidth(400)
            editor.setText(form_text)
            editor.setReadOnly(True)
            editor.setFocusPolicy(Qt.NoFocus)
            editor.setStyleSheet("background-color : linen; color: black;")

        Form_layout.addRow(label, editor)

    #################################################################
    def make_layout(self):
        if (
            self.parent.info_dict["componentType"] == "MODULE"
            or self.parent.info_dict["componentType"] == "practice"
        ):
            return self.layout_ModuleQC()
        return self.layout_ModuleQC()

    def layout_ModuleQC(self):
        HBox = QHBoxLayout()

        Form_layout = QFormLayout()

        for k, v in self.parent.testRun.items():
            if k not in ["results"]:
                self.add_info(Form_layout, f"{k[:1].upper()+k[1:]} :", str(v))

        for container in ["results"]:
            if container not in self.parent.testRun:
                continue

            keylist = []

            for k, v in self.parent.testRun.get(container).items():
                if k not in [
                    "results",
                    "metadata",
                    "Metadata",
                    "Measurements",
                    "properties",
                    "property",
                ]:
                    if k not in keylist:
                        self.add_info(Form_layout, f"{k[:1].upper()+k[1:]} :", str(v))
                        keylist.append(k)
                else:
                    for k2, v2 in self.parent.testRun.get(container).get(k).items():
                        if k2 not in keylist:
                            self.add_info(
                                Form_layout, f"{k2[:1].upper()+k2[1:]} :", str(v2)
                            )
                            keylist.append(k2)

        HBox.addLayout(Form_layout)

        return HBox



from __future__ import annotations

from pathlib import Path

from PyQt5.QtCore import Qt
from PyQt5.QtGui import QImage, QPixmap
from PyQt5.QtWidgets import (
    QHBoxLayout,
    QLabel,
    QLineEdit,
    QPushButton,
    QScrollArea,
    QTextEdit,
    QVBoxLayout,
    QWidget,
)


class ConfirmWindow(QWidget):
    def __init__(self, parent=None):
        super(QWidget, self).__init__(parent)
        self.parent = parent

        titlebox = QVBoxLayout()
        layout = QVBoxLayout()
        button_box = QHBoxLayout()
        inner_box = QVBoxLayout()
        image_box = QVBoxLayout()

        label_title = QLabel()
        label_title.setText(
            '<center><font size="5">Confirm before uploading to the database</font></center>'
        )
        label_practice = QLabel()
        label_practice.setText(
            '<center><font size="4" color = "green"> Practice Mode</font></center>'
        )

        Upload_button = QPushButton("&Upload!")
        Upload_button.clicked.connect(self.upload_to_db)
        json_button = QPushButton("&Check json (for expert)")
        json_button.clicked.connect(self.check_json)
        back_button = QPushButton("&Back")
        back_button.clicked.connect(self.back_page)

        titlebox.addWidget(label_title)

        button_box.addWidget(back_button)
        button_box.addStretch()
        button_box.addWidget(json_button)
        button_box.addWidget(Upload_button)

        inner = QScrollArea()
        inner.setFixedWidth(600)
        inner.setFixedHeight(400)
        result_wid = QWidget()
        result_wid.setLayout(self.layout_moduleQC())

        inner.setWidgetResizable(True)
        inner.setWidget(result_wid)
        inner_box.addWidget(inner)

        self.doppo = str(Path(__file__).resolve().parent / "flatness_map.png")
        image = QImage(self.doppo)
        self.width = 470
        self.height = 470
        image = image.scaled(
            self.width, self.height, Qt.KeepAspectRatio, Qt.FastTransformation
        )
        imageLabel = QLabel()
        imageLabel.setPixmap(QPixmap.fromImage(image))
        imageLabel.scaleFactor = 1.0
        image_box.addWidget(imageLabel)

        image_box_h = QHBoxLayout()
        image_box_h.addStretch()
        image_box_h.addLayout(image_box)
        image_box_h.addStretch()

        layout.addLayout(titlebox)
        layout.addLayout(inner_box)
        layout.addLayout(button_box)
        layout.addLayout(image_box_h)

        self.setLayout(layout)

    def back_page(self):
        self.parent.back_to_test()

    def check_json(self):
        self.parent.confirm_json()

    def upload_to_db(self):
        self.parent.upload_to_db()

    def add_info(self, Form_layout, label_str, form_text):
        label = QLabel()
        label.setText(label_str)

        if label_str == "Comment :":
            inner = QTextEdit()
            inner.setText(form_text)
            inner.setReadOnly(True)
            inner.setFocusPolicy(Qt.NoFocus)
            inner.setStyleSheet("background-color : linen; color: black;")

            editor = QScrollArea()
            editor.setWidgetResizable(True)
            editor.setWidget(inner)

        else:
            editor = QLineEdit()
            editor.setText(str(form_text))
            editor.setReadOnly(True)
            editor.setFocusPolicy(Qt.NoFocus)
            editor.setStyleSheet("background-color : linen; color: black;")
        #        editor.setStyleSheet("background-color : azure")

        Form_layout.addRow(label, editor)

    #################################################################

    def layout_ModuleQC(self):
        HBox = QHBoxLayout()

        Form_layout = self.parent.confirm_layout_common(self)

        self.add_info(
            Form_layout,
            "Back-side Flatness :",
            self.parent.testRun["results"]["BACKSIDE_FLATNESS"],
        )
        self.add_info(
            Form_layout,
            "Angles :",
            self.parent.testRun["results"]["ANGLES"],
        )
        self.add_info(
            Form_layout, "Comment :", self.parent.testRun["results"]["comment"]
        )

        HBox.addLayout(Form_layout)

        return HBox



from __future__ import annotations

import logging

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import (
    QGridLayout,
    QHBoxLayout,
    QLabel,
    QLineEdit,
    QPushButton,
    QVBoxLayout,
    QWidget,
)

log = logging.getLogger(__name__)


class ComponentInfoWindow(QWidget):
    def __init__(self, parent=None):
        super(QWidget, self).__init__(parent)
        self.parent = parent

        layout = QVBoxLayout()

        label_text = QLabel()
        label_text.setText(
            '<center><font size="5">Component Information</font></center>'
        )

        Next_button = QPushButton("&Next")
        Next_button.clicked.connect(self.go_to_qc)
        Back_button = QPushButton("&Back")
        Back_button.clicked.connect(self.back_page)

        layout.addWidget(label_text)

        grid_layout = self.make_infotable()
        hbox = QHBoxLayout()
        hbox.addLayout(grid_layout)

        layout.addStretch()
        layout.addLayout(hbox)
        layout.addStretch()
        self.add_button(layout, Back_button, Next_button)
        self.setLayout(layout)

    def add_button(self, vlayout, back_button, next_button):
        hbox = QHBoxLayout()

        hbox.addWidget(back_button)
        hbox.addStretch()
        hbox.addWidget(next_button)

        vlayout.addLayout(hbox)

    def add_infotable(self, grid_layout, label_str, info_str, form_size, i):
        label_text = QLabel()
        label_text.setText('<font size="4">' + label_str + "</font>")

        info_text = QLineEdit()
        info_text.setText(info_str)
        info_text.setReadOnly(True)
        info_text.setFocusPolicy(Qt.NoFocus)
        info_text.setStyleSheet("background-color : linen;")
        info_text.setFixedHeight(label_text.sizeHint().height() + 3)
        info_text.setFixedWidth(form_size)

        grid_layout.setSpacing(3)
        grid_layout.addWidget(label_text, i, 0)
        grid_layout.addWidget(info_text, i, 1)

    def get_maximum(self, contents_dict):
        N_char = []
        for key in contents_dict:
            label = QLabel()
            label.setText(contents_dict[key])
            N_char.append(label.sizeHint().width())
        return max(N_char)

    def make_infotable(self):
        log.info(self.parent.info_dict)
        contents = {
            "ATLAS Serial Number:": self.parent.info_dict["component"],
            "Component Type:": self.parent.info_dict["componentType"],
            "Current Stage:": self.parent.info_dict["currentStage"],
        }

        form_size = max([170, self.get_maximum(contents) + 10])

        grid_layout = QGridLayout()
        self.add_infotable(
            grid_layout,
            "ATLAS Serial Number:",
            self.parent.info_dict["component"],
            form_size,
            0,
        )
        self.add_infotable(
            grid_layout,
            "Component Type:",
            self.parent.info_dict["componentType"],
            form_size,
            1,
        )
        self.add_infotable(
            grid_layout,
            "Current Stage:",
            self.parent.info_dict["currentStage"],
            form_size,
            5,
        )

        return grid_layout

    def go_to_qc(self):
        self.parent.moduleQC_choose_test()

    def back_page(self):
        self.parent.back_from_componentinfo()



from __future__ import annotations

import logging

from PyQt5.QtWidgets import QMainWindow

from module_qc_nonelec_gui.qc_tests.IV_MEASURE_BARE import user_input

log = logging.getLogger(__name__)


class TestWindow(QMainWindow):
    ############################################################################################
    def __init__(self, parent=None):
        super(QMainWindow, self).__init__()
        self.parent = parent

        self.setWindowTitle("Sensor I-V")
        self.scale_window(510, 255)

        self.result_info = {"comment": "", "filename": ""}

        self.initial_wid = None

    ###########################################################################################

    def init_ui(self):
        self.initial_wid = user_input.InitialWindow(self)
        self.parent.update_widget(self.initial_wid)

    def scale_window(self, x, y):
        self.setGeometry(0, 0, x, y)

    def close_and_return(self):
        self.close()
        self.parent.back_from_test()

    def back_page(self):
        self.parent.init_ui()

    def back_window(self):
        self.parent.receive_backpage()

    def call_another_window(self, window):
        self.hide()
        window.init_ui()



from __future__ import annotations

import json
import logging
import os
from pathlib import Path

log = logging.getLogger(__name__)


class config_modifier:
    def __init__(self):
        self.module_qc_nonelec_gui_dir = "/".join(
            os.path.realpath(__file__).split("/")[:-2]
        )
        self.module_qc_nonelec_gui_config = (
            self.module_qc_nonelec_gui_dir + "/configuration/configuration.json"
        )

        with Path(self.module_qc_nonelec_gui_config).open(encoding="utf-8") as f_conf:
            self.configuration = json.load(f_conf)

    def print_current(self):
        # Print current setting of LocalDB
        log.info(f"Config File : {self.module_qc_nonelec_gui_config}")
        log.info(f"LocalDB IP  : {self.configuration['localDB_info']['address']}")
        log.info(f"LocalDB Port: {self.configuration['localDB_info']['port']}")

    def set_ip_port(self, ip="127.0.0.1", port="27017"):
        self.configuration["localDB_info"]["address"] = ip
        self.configuration["localDB_info"]["port"] = port

    def wrap_up(self, filename=""):
        if filename == "":
            filename = self.module_qc_nonelec_gui_config
        with Path(filename).open(mode="w", encoding="utf-8") as f_new:
            json.dump(self.configuration, f_new, indent=4, ensure_ascii=False)



from __future__ import annotations

from PyQt5.QtWidgets import QMainWindow

from module_qc_nonelec_gui.qc_tests.WIREBOND_PULL_TEST import user_input


class TestWindow(QMainWindow):
    ############################################################################################
    def __init__(self, parent=None):
        super(QMainWindow, self).__init__()
        self.parent = parent
        self.initial_wid = None

        self.setWindowTitle("Wirebond pull tests")
        self.scale_window(340, 255)

        self.result_info = {
            "minimum_load": "",
            "minimum_load_unit": "g",
            "maximum_load": "",
            "maximum_load_unit": "g",
            "mean_load": "",
            "mean_load_unit": "g",
            "load_standard_deviation": "",
            "load_standard_deviation_unit": "g",
            "percentage_of_heel_breaks": "",
            "percentage_of_heel_breaks_unit": "%",
            "comment": "",
        }

    ###########################################################################################

    def init_ui(self):
        self.initial_wid = user_input.InitialWindow(self)
        self.parent.update_widget(self.initial_wid)

    def scale_window(self, x, y):
        self.setGeometry(0, 0, x, y)

    def close_and_return(self):
        self.close()
        self.parent.back_from_test()

    def back_page(self):
        self.parent.init_ui()

    def back_window(self):
        self.parent.receive_backpage()

    def return_result(self):
        self.parent.receive_result(self, self.test_result_dict)



from __future__ import annotations

from PyQt5.QtCore import Qt
from PyQt5.QtGui import QDoubleValidator
from PyQt5.QtWidgets import (
    QGridLayout,
    QHBoxLayout,
    QLabel,
    QLineEdit,
    QMessageBox,
    QPushButton,
    QScrollArea,
    QTextEdit,
    QVBoxLayout,
    QWidget,
)


class InitialWindow(QWidget):
    def __init__(self, parent=None):
        super(QWidget, self).__init__(parent)
        self.parent = parent
        self.edit_d = None
        self.edit_h = None
        self.edit_m = None
        self.edit_s = None

        layout = QVBoxLayout()
        bottom_box = QHBoxLayout()
        grid_edit = QGridLayout()

        label_title = QLabel()
        label_title.setText(
            '<center><font size="7">Input Glue Information</font></center>'
        )

        Next_button = QPushButton("&Next")
        Next_button.clicked.connect(self.pass_result)
        Back_button = QPushButton("&Back")
        Back_button.clicked.connect(self.back_page)

        bottom_box.addWidget(Back_button)
        bottom_box.addStretch()
        bottom_box.addWidget(Next_button)

        label_comment = QLabel()
        label_comment.setText("comment : ")
        label_roomTemp = QLabel()
        label_roomTemp.setText("Room temperature : ")
        label_humidity = QLabel()
        label_humidity.setText("Humidity : ")
        label_adhesion = QLabel()
        label_adhesion.setText("Adhesion time : ")

        self.edit_comment = QTextEdit()
        self.edit_comment.setText(self.parent.result_info["comment"])

        self.edit_glue = self.make_textform(
            "Glue name : ", self.parent.result_info["glue_name"], grid_edit, 0
        )
        self.edit_ratio = self.make_textform(
            "Volume ratio of glue mixture : ",
            self.parent.result_info["ratio"],
            grid_edit,
            1,
        )
        self.edit_batch = self.make_textform(
            "Glue batch number : ", self.parent.result_info["batch"], grid_edit, 2
        )

        grid_edit.addWidget(label_roomTemp, 3, 0)
        grid_edit.addLayout(self.make_roomTemp_layout(), 3, 1)

        grid_edit.addWidget(label_humidity, 4, 0)
        grid_edit.addLayout(self.make_humidity_layout(), 4, 1)

        #        grid_edit.addWidget(label_adhesion,5,0)
        #        grid_edit.addLayout(self.make_timeform_layout(),5,1)
        grid_edit.addWidget(label_comment, 5, 0)

        outer = QScrollArea()
        outer.setWidgetResizable(True)
        outer.setWidget(self.edit_comment)
        grid_edit.addWidget(outer, 5, 1)

        layout.addWidget(label_title)
        layout.addStretch()
        layout.addLayout(grid_edit)
        layout.addLayout(bottom_box)

        self.setLayout(layout)

    def make_timeform_layout(self):
        grid_adhesion = QGridLayout()

        label_d = QLabel()
        label_d.setText("d")
        label_h = QLabel()
        label_h.setText("h")
        label_m = QLabel()
        label_m.setText("m")
        label_s = QLabel()
        label_s.setText("s")

        self.edit_d = QLineEdit()
        self.edit_d.setAlignment(Qt.AlignRight)
        self.edit_d.setText(str(self.parent.result_info["adhesion"]["day"]))
        self.edit_h = QLineEdit()
        self.edit_h.setAlignment(Qt.AlignRight)
        self.edit_h.setText(str(self.parent.result_info["adhesion"]["hour"]))
        self.edit_m = QLineEdit()
        self.edit_m.setAlignment(Qt.AlignRight)
        self.edit_m.setText(str(self.parent.result_info["adhesion"]["minute"]))
        self.edit_s = QLineEdit()
        self.edit_s.setAlignment(Qt.AlignRight)
        self.edit_s.setText(str(self.parent.result_info["adhesion"]["second"]))

        grid_adhesion.addWidget(self.edit_d, 0, 0)
        grid_adhesion.addWidget(label_d, 0, 1)
        grid_adhesion.addWidget(self.edit_h, 0, 2)
        grid_adhesion.addWidget(label_h, 0, 3)
        grid_adhesion.addWidget(self.edit_m, 0, 4)
        grid_adhesion.addWidget(label_m, 0, 5)
        grid_adhesion.addWidget(self.edit_s, 0, 6)
        grid_adhesion.addWidget(label_s, 0, 7)

        return grid_adhesion

    def make_roomTemp_layout(self):
        grid_layout, self.edit_roomTemp = self.make_value_unit_layout(
            str(self.parent.result_info["Temp_value"]),
            self.parent.result_info["Temp_unit"],
        )

        return grid_layout

    def make_humidity_layout(self):
        grid_layout, self.edit_humidity = self.make_value_unit_layout(
            str(self.parent.result_info["Humidity_value"]),
            self.parent.result_info["Humidity_unit"],
        )

        return grid_layout

    def make_value_unit_layout(self, value_str, unit_str):
        grid_layout = QGridLayout()

        label = QLabel()
        label.setText(unit_str)

        edit_value = QLineEdit()
        edit_value.setValidator(QDoubleValidator())
        edit_value.setAlignment(Qt.AlignRight)
        edit_value.setText(value_str)

        grid_layout.addWidget(edit_value, 0, 0)
        grid_layout.addWidget(label, 0, 1)

        return grid_layout, edit_value

    def make_textform(self, label, initial, layout, i):
        label_text = QLabel()
        label_text.setText(label)
        edit_text = QLineEdit()
        edit_text.setText(initial)

        if initial == "TBA":
            edit_text.setStyleSheet("color: black; background-color:linen;")
            edit_text.setReadOnly(True)
            edit_text.setFocusPolicy(Qt.NoFocus)

        layout.addWidget(label_text, i, 0)
        layout.addWidget(edit_text, i, 1)

        return edit_text

    def pass_result(self):
        token = 0
        try:
            result_dict = {
                "comment": self.edit_comment.toPlainText(),
                "glue_name": self.edit_glue.text(),
                "ratio": self.edit_ratio.text(),
                "batch": self.edit_batch.text(),
                "Temp_value": float(self.edit_roomTemp.text()),
                "Humidity_value": float(self.edit_humidity.text()),
                #                'adhesion'   :{
                #                    'day'        :int( self.edit_d.text() ),
                #                    'hour'       :int( self.edit_h.text() ),
                #                    'minute'     :int( self.edit_m.text() ),
                #                    'second'     :int( self.edit_s.text() )
                #                }
            }
            #            if int( self.edit_d.text() )<0 or int( self.edit_h.text() ) <0 or int( self.edit_m.text() )<0 or int( self.edit_s.text() )<0:
            #                QMessageBox.warning(None, 'Warning','Adhesion time should be positive value.', QMessageBox.Ok)
            #            elif int( self.edit_h.text() )>=24 or int( self.edit_m.text() )>=60 or int( self.edit_s.text() )>=60:
            #                QMessageBox.warning(None, 'Warning','Please round up the time values .', QMessageBox.Ok)
            #            else:
            #                token=1
            token = 1
        except Exception:
            #            import traceback
            #            print (traceback.format_exc() )
            QMessageBox.warning(
                None, "Warning", "Please Fill these form.", QMessageBox.Ok
            )

        if token == 1:
            if (
                float(self.edit_roomTemp.text()) < -50
                or float(self.edit_roomTemp.text()) > 50
            ):
                QMessageBox.warning(
                    None,
                    "Warning",
                    "Hey! You alive? \nIt is too difficult to live at this room temperature.",
                    QMessageBox.Ok,
                )
            else:
                if (
                    float(self.edit_humidity.text()) < 0
                    or float(self.edit_humidity.text()) > 100
                ):
                    QMessageBox.warning(
                        None,
                        "Warning",
                        "It is impossible humidity value.",
                        QMessageBox.Ok,
                    )
                else:
                    self.parent.receive_result(result_dict)

    def back_page(self):
        self.parent.close_and_return()



from __future__ import annotations

import logging

from PyQt5.QtWidgets import QMainWindow

from module_qc_nonelec_gui.qc_tests.glue_info_module_flex import initial_Glue_info

log = logging.getLogger(__name__)


class TestWindow(QMainWindow):
    ############################################################################################
    def __init__(self, parent=None):
        super(QMainWindow, self).__init__()
        self.parent = parent
        self.test_result_dict = None
        self.initial_wid = None

        self.setWindowTitle("Glue information Module Flex Attach")

        self.result_info = {
            "comment": "",
            "glue_name": "Araldite 2011",
            "ratio": "A:1,B:1",
            "batch": "TBA",
            "Temp_value": "",
            "Temp_unit": "degC",
            "Humidity_value": "",
            "Humidity_unit": "%RH",
            "adhesion": {"day": 0, "hour": 10, "minute": 0, "second": 0},
        }

    def receive_result(self, result):
        self.result_info["glue_name"] = result["glue_name"]
        self.result_info["ratio"] = result["ratio"]
        self.result_info["batch"] = result["batch"]
        self.result_info["Temp_value"] = result["Temp_value"]
        self.result_info["Humidity_value"] = result["Humidity_value"]
        #        self.result_info['adhesion']['day']    = result['adhesion']['day']
        #        self.result_info['adhesion']['hour']   = result['adhesion']['hour']
        #        self.result_info['adhesion']['minute'] = result['adhesion']['minute']
        #        self.result_info['adhesion']['second'] = result['adhesion']['second']
        self.result_info["comment"] = result["comment"]

        self.fill_result()
        self.return_result()

    def fill_result(self):
        self.test_result_dict = {
            "results": {
                "localDB": {
                    "property": {
                        "Glue_name": self.result_info["glue_name"],
                        "Volume_ratio_of_glue_mixture": self.result_info["ratio"],
                        "Glue_batch_number": self.result_info["batch"],
                    },
                    "Room_temperature": self.result_info["Temp_value"],
                    "Temperature_unit": self.result_info["Temp_unit"],
                    "Humidity": self.result_info["Humidity_value"],
                    "Humidity_unit": self.result_info["Humidity_unit"],
                    #                    'Adhesion_Time'    :self.result_info['adhesion'],
                    "comment": self.result_info["comment"],
                },
                "ITkPD": {
                    "property": {
                        "Glue_name": self.result_info["glue_name"],
                        "Volume_ratio_of_glue_mixture": self.result_info["ratio"],
                        "Glue_batch_number": self.result_info["batch"],
                    },
                    "Room_temperature": self.result_info["Temp_value"],
                    "Temperature_unit": self.result_info["Temp_unit"],
                    "Humidity": self.result_info["Humidity_value"],
                    "Humidity_unit": self.result_info["Humidity_unit"],
                    #                    'Adhesion_Time'    :self.result_info['adhesion'],
                    "comment": self.result_info["comment"],
                },
                "summary": {
                    "property": {
                        "Glue_name": self.result_info["glue_name"],
                        "Volume_ratio_of_glue_mixture": self.result_info["ratio"],
                        "Glue_batch_number": self.result_info["batch"],
                    },
                    "Room_temperature": self.result_info["Temp_value"],
                    "Temperature_unit": self.result_info["Temp_unit"],
                    "Humidity": self.result_info["Humidity_value"],
                    "Humidity_unit": self.result_info["Humidity_unit"],
                    #                    'Adhesion_Time'    :self.result_info['adhesion'],
                    "comment": self.result_info["comment"],
                },
            }
        }
        log.info(
            "[Test Result]  "
            + str(self.test_result_dict["results"]["localDB"]["property"]["Glue_name"])
        )

    ###########################################################################################

    def init_ui(self):
        self.initial_wid = initial_Glue_info.InitialWindow(self)
        self.update_widget(self.initial_wid)

    def scale_window(self, x, y):
        self.setGeometry(0, 0, x, y)

    def update_widget(self, w):
        self.scale_window(480, 360)
        self.setCentralWidget(w)
        self.show()

    def close_and_return(self):
        self.close()
        self.parent.back_from_test()

    def back_page(self):
        self.parent.init_ui()

    def back_window(self):
        self.parent.receive_backpage()

    def call_another_window(self, window):
        self.hide()
        window.init_ui()

    def return_result(self):
        self.parent.receive_result(self, self.test_result_dict)



from __future__ import annotations

from PyQt5.QtWidgets import (
    QFormLayout,
    QHBoxLayout,
    QLabel,
    QLineEdit,
    QPushButton,
    QVBoxLayout,
    QWidget,
)


class ConnectPDWindow(QWidget):
    def __init__(self, parent=None):
        super(QWidget, self).__init__()
        self.parent = parent

        titlebox = QVBoxLayout()
        layout = QVBoxLayout()
        hbox_bottom = QHBoxLayout()
        edit_form = QFormLayout()
        # form_box = QHBoxLayout()

        label_text = QLabel()
        label_text.setText(
            '<center><font size="6">Log in to Production DB</font></center>'
        )
        label_accesscode1 = QLabel()
        label_accesscode1.setText("ITKDB ACCESS CODE 1:")
        label_accesscode2 = QLabel()
        label_accesscode2.setText("ITKDB ACCESS CODE 2:")

        self.edit_accesscode1 = QLineEdit()
        self.edit_accesscode1.setEchoMode(QLineEdit().Password)
        self.edit_accesscode2 = QLineEdit()
        self.edit_accesscode2.setEchoMode(QLineEdit().Password)

        Next_button = QPushButton("&Login")
        Next_button.clicked.connect(self.pass_accesscode)

        titlebox.addWidget(label_text)

        hbox_bottom.addWidget(Next_button)

        edit_form.addRow(label_accesscode1, self.edit_accesscode1)
        edit_form.addRow(label_accesscode2, self.edit_accesscode2)

        layout.addStretch()
        layout.addLayout(titlebox)
        layout.addStretch()
        layout.addLayout(edit_form)
        layout.addLayout(hbox_bottom)

        self.setLayout(layout)

    def pass_accesscode(self):
        self.parent.connectpd(
            self.edit_accesscode1.text(), self.edit_accesscode2.text()
        )



from __future__ import annotations

from PyQt5.QtWidgets import (
    QGridLayout,
    QLabel,
    QLineEdit,
    QMessageBox,
    QPushButton,
    QWidget,
)


class SensorInfoWindow(QWidget):
    def __init__(self, parent=None):
        super(QWidget, self).__init__()
        self.parent = parent

        layout = QGridLayout()

        label_title = QLabel()
        label_title.setText("Input Sensor ATLAS Serial Number:")

        label_sensor = QLabel()
        label_sensor.setText("ATLAS Serial Number")

        self.edit_sensor = QLineEdit()

        button_next = QPushButton("&Assemble")
        button_next.clicked.connect(self.next_page)
        button_back = QPushButton("&Back")
        button_back.clicked.connect(self.back_page)
        button_skip = QPushButton("&Skip")
        button_skip.clicked.connect(self.skip_page)

        layout.addWidget(label_title, 0, 0)
        layout.addWidget(label_sensor, 1, 0)
        layout.addWidget(self.edit_sensor, 1, 1)
        layout.addWidget(button_back, 2, 0)
        layout.addWidget(button_skip, 2, 1)
        layout.addWidget(button_next, 2, 2)

        self.setLayout(layout)

    def next_page(self):
        self.parent.sensor_id = self.edit_sensor.text()
        if self.parent.sensor_id != "":
            try:
                doc = self.parent.get_componentinfo(self.parent.sensor_id)
            except Exception:
                QMessageBox.warning(
                    None, "Error", "Cannot get Sensor Tile information", QMessageBox.Ok
                )
            parents_list = doc["parents"]
            isAssembled = False
            if parents_list is not None:
                for parent in parents_list:
                    if (
                        parent["componentType"]["code"] == "BARE_MODULE"
                        and parent["history"][-1]["action"] == "assembly"
                    ):
                        isAssembled = True
                        QMessageBox.critical(
                            None,
                            "Error",
                            self.parent.sensor_id + " is already assembled.",
                            QMessageBox.Ok,
                        )
            else:
                if not isAssembled:
                    self.parent.assemble_sensor()
                    self.parent.choose_menu()
        else:
            QMessageBox.warning(None, "Warning", "Input Sensor ID", QMessageBox.Ok)

    def back_page(self):
        self.parent.feinfo()

    def skip_page(self):
        self.parent.choose_menu()



from __future__ import annotations

import contextlib

from PyQt5.QtWidgets import (
    QCheckBox,
    QGridLayout,
    QLabel,
    QLineEdit,
    QPushButton,
    QVBoxLayout,
)


def commonpart_gen(w):
    layout = QGridLayout()
    label_comment = QLabel()
    label_comment.setText("Comment:")
    layout.addWidget(label_comment, 0, 0, 1, 2)
    w.edit_comment = QLineEdit()
    layout.addWidget(w.edit_comment, 1, 0, 2, 2)
    button_checked = QPushButton("Checked")
    button_checked.clicked.connect(w.checked)
    layout.addWidget(button_checked, 3, 1)

    button_back = QPushButton("Back")
    button_back.clicked.connect(w.back_page)
    button_next = QPushButton("Next")
    button_next.clicked.connect(w.next_page)
    layout.addWidget(button_back, 4, 0)
    layout.addWidget(button_next, 4, 1)

    return layout


def checklist_gen(w, page, tot_page):
    layout = QVBoxLayout()
    layout_child = QGridLayout()
    label_page = QLabel()
    label_page.setText(f"page: {page + 1}/{tot_page}")
    layout.addWidget(label_page)
    label = QLabel()
    label.setText("If there are any problems, check in checkbox.")
    layout.addWidget(label)

    w.cb = {}
    list_keys = list(w.parent.checklist_dict[str(page)].keys())
    for index_key, key in enumerate(list_keys):
        items = w.parent.checklist_dict[str(page)][key]
        w.cb[index_key] = {}
        for index, item in enumerate(items):
            w.cb[index_key][index] = QCheckBox(item, w)
            # w.cb[index].stateChanged.connect(lambda: w.checkBoxChangeAction(w.cb[index].checkState(),item))
            # print(type(item))
            layout_child.addWidget(w.cb[index_key][index], index, index_key)
            with contextlib.suppress(Exception):
                if item in w.parent.anomaly_dic[str(page)]:
                    w.cb[index_key][index].toggle()
    layout.addLayout(layout_child)
    layout.addLayout(commonpart_gen(w))

    return layout



from __future__ import annotations

import logging

from PyQt5.QtWidgets import (
    QGridLayout,
    QLabel,
    QLineEdit,
    QMessageBox,
    QPushButton,
    QWidget,
)

log = logging.getLogger(__name__)


class BareInfoWindow(QWidget):
    def __init__(self, parent=None):
        super(QWidget, self).__init__()
        self.parent = parent

        layout = QGridLayout()

        label_title = QLabel()
        label_title.setText("Input Bare Module ATLAS Serial Number:")

        label_bare = QLabel()
        label_bare.setText("ATLAS Serial Number")

        self.edit_bare = QLineEdit()

        button_next = QPushButton("&Next")
        button_next.clicked.connect(self.next_page)
        button_back = QPushButton("&Back")
        button_back.clicked.connect(self.back_page)

        layout.addWidget(label_title, 0, 0)
        layout.addWidget(label_bare, 1, 0)
        layout.addWidget(self.edit_bare, 1, 1)
        layout.addWidget(button_back, 2, 0)
        layout.addWidget(button_next, 2, 1)

        self.setLayout(layout)

    def next_page(self):
        try:
            self.parent.get_bareinfo(self.edit_bare.text())
        except Exception:
            QMessageBox.warning(
                None, "Warning", "Cannot get bare module information", QMessageBox.Ok
            )
            self.parent.bareupdate()
        # log.info(self.parent.bare_doc)
        children_list = self.parent.bare_doc["children"]
        # isSensor = False
        # isFEChip = False
        for child in children_list:
            if (
                child["componentType"]["code"] == "FE_CHIP"
                and child["component"] is not None
            ):
                # isFEChip = True
                # msgBox = QMessageBox.warning(None, 'Warning',"FE chip is already assembled.", QMessageBox.Ok)
                QMessageBox.warning(
                    None,
                    "Warning",
                    "FE chip "
                    + child["component"]["serialNumber"]
                    + " is already assembled.",
                    QMessageBox.Ok,
                )
                log.info(child["component"])
            elif (
                child["componentType"]["code"] == "SENSOR_TILE"
                and child["component"] is not None
            ):
                # isSensor = True
                # msgBox = QMessageBox.warning(None, 'Warning',"Sensor tile is already assembled.", QMessageBox.Ok)
                QMessageBox.warning(
                    None,
                    "Warning",
                    "Sensor tile "
                    + child["component"]["serialNumber"]
                    + " is already assembled.",
                    QMessageBox.Ok,
                )
        self.parent.feinfo()

    def back_page(self):
        self.parent.choose_menu()



from __future__ import annotations

import logging

from PyQt5.QtWidgets import QGridLayout, QLabel, QPushButton, QRadioButton, QWidget

log = logging.getLogger(__name__)


class InitialWindow(QWidget):
    def __init__(self, parent=None):
        super(QWidget, self).__init__()
        self.parent = parent

        layout = QGridLayout()
        self.menu = "NULL"

        label_text = QLabel()
        label_text.setText("Choose Menu")

        self.rb_bareregist = QRadioButton("Register Bare Module")
        self.rb_bareupdate = QRadioButton("Update Bare Module Info.")

        self.rb_bareregist.toggled.connect(self.onClicked)
        self.rb_bareupdate.toggled.connect(self.onClicked)

        button_next = QPushButton("&Next")
        button_next.clicked.connect(self.pass_menu)
        button_back = QPushButton("&Back")
        button_back.clicked.connect(self.back)

        layout.addWidget(label_text, 0, 0, 1, 2)
        layout.addWidget(self.rb_bareregist, 1, 0, 1, 2)
        layout.addWidget(self.rb_bareupdate, 2, 0, 1, 2)
        layout.addWidget(button_next, 3, 1, 1, 1)
        layout.addWidget(button_back, 3, 0, 1, 1)

        self.setLayout(layout)

    def onClicked(self):
        radiobtn = self.sender()
        if radiobtn.isChecked():
            if radiobtn.text() == "Register Bare Module":
                self.menu = "Register"
            elif radiobtn.text() == "Update Bare Module Info.":
                self.menu = "Update"

    def pass_menu(self):
        if self.menu == "Register":
            self.parent.bareregist()
        elif self.menu == "Update":
            self.parent.bareupdate()
        elif self.menu == "NULL":
            log.info("Please choose menu")

    def back(self):
        self.parent.parent.call_selectmode()



from __future__ import annotations

import copy
import json
import logging
import pprint
import sys
import traceback
from datetime import datetime
from importlib import machinery
from pathlib import Path

import pymongo
import requests
from bson.objectid import ObjectId
from PyQt5.QtWidgets import QMessageBox

logging.basicConfig(stream=sys.stdout, level=logging.INFO)
log = logging.getLogger(__name__)


def get_sensor(self, component):
    ID = self.localDB.component.find_one({"serialNumber": component}, {"_id": 1})
    SN = None
    for i in self.localDB.childParentRelation.find({"parent": str(ID["_id"])}):
        child_id = i["child"]
        SN = self.localDB.component.find_one(
            {"componentType": "sensor_tile", "_id": ObjectId(child_id)},
            {"serialNumber": 1, "_id": 0},
        )
        if SN:
            return SN["serialNumber"]
    return None


def fill_info_sensor(self, number, info_dict_bare):
    info_dict_sen = copy.copy(info_dict_bare)
    component = self.localDB.component.find_one({"serialNumber": number})
    qcStatus = self.localDB.QC.module.status.find_one(
        {"component": str(component["_id"])}
    )
    qcStages = self.localDBtools.QC.stages.find_one({"code": "SENSOR_TILE"})

    info_dict_sen["component"] = number
    info_dict_sen["componentType"] = "SENSOR_TILE"
    info_dict_sen["FE_version"] = None
    info_dict_sen["currentStage"] = qcStatus["currentStage"]
    info_dict_sen["currentStageCode"] = qcStages["stages"].get(qcStatus["currentStage"])
    return info_dict_sen


def read_data(filename):
    volt = []
    curr = []
    cap = []
    D = []
    temp = []
    time = []
    with Path(filename).open(encoding="utf-8") as f:
        isDataLine = False
        isNew_dat = False
        lines = f.readlines()
        # for line in lines:
        #    if line[39:43] == "temp":
        #        isNew_dat = True
        for line in lines:
            if line[0:10] == "voltage[V]":
                isDataLine = True
                continue
            if isDataLine:
                rawdata = line.split(" ")
                if len(rawdata) == 7:
                    volt.append(float(rawdata[0]) * (-1))
                    curr.append(float(rawdata[1]) * (10**6) * (-1))
                    cap.append(float(rawdata[2]))
                    D.append(float(rawdata[3]))
                    temp.append(float(rawdata[4]))
                    timedata = rawdata[6].split()
                    time.append(timedata[0])
                elif len(rawdata) == 6:
                    volt.append(float(rawdata[0]) * (-1))
                    curr.append(float(rawdata[1]) * (10**6) * (-1))
                    cap.append(float(rawdata[2]))
                    D.append(float(rawdata[3]))
                    timedata = rawdata[5].split()
                    time.append(timedata[0])
            # if isDataLine and isNew_dat:
            #    rawdata = line.split(" ")
            #    volt.append(float(rawdata[0]) * (-1))
            #    curr.append(float(rawdata[1]) * (10**6) * (-1))
            #    cap.append(float(rawdata[2]))
            #    D.append(float(rawdata[3]))
            #    temp.append(float(rawdata[4]))
            #    timedata = rawdata[6].replace(":", "")
            #    time.append(float(timedata))

            # elif isDataLine and (not isNew_dat):
            #    rawdata = line.split(" ")
            #    volt.append(float(rawdata[0]) * (-1))
            #    curr.append(float(rawdata[1]) * (10**6) * (-1))
            #    cap.append(float(rawdata[2]))
            #    D.append(float(rawdata[3]))
            #    timedata = rawdata[5].replace(":", "")
            #    time.append(float(timedata))

    if not isNew_dat:
        temp = [20] * len(volt)
    base_time = datetime.strptime(time[0], "%H:%M:%S")
    pass_time = []
    for t in time:
        elapsed_time = (datetime.strptime(t, "%H:%M:%S") - base_time).total_seconds()
        pass_time.append(int(elapsed_time))

    data = {
        "voltage": volt,
        "current": curr,
        "cap": cap,
        "D": D,
        "temp": temp,
        "time": pass_time,
    }

    log.info(f"get from dat file : {data}")
    return data


def get_testRun(self, name, version, serialNumber):
    testRunFormat = self.localDBtools.QC.tests.find_one(
        {
            "code": "IV_MEASURE",
            "componentType.code": "SENSOR_TILE",
        }
    )
    testRun = {
        "serialNumber": name,
        "testType": "IV_MEASURE",
        "results": {
            "property": {prop["code"]: None for prop in testRunFormat["properties"]},
            "comment": "",
            "metadata": {
                "QCHELPER_VERSION": version,
                "MODULE_SN": serialNumber,
            },
            "Metadata": {},
            "Measurements": {},
        },
    }
    testRun.get("results").update(
        {param["code"]: None for param in testRunFormat["parameters"]}
    )
    return testRun


def receive_result_IV(MainWindow, InitialWindow):
    InitialWindow.hide()
    confirm_IV_result(MainWindow)


def confirm_IV_result(self):
    log.info("==========================================================")
    log.info("confirm_IV_result(): [results] sensor!!!")
    log.info(pprint.pformat(self.testRun_sen))
    log.info("==========================================================")

    try:
        test_confirm = "qc_tests/IV_MEASURE_BARE/confirm.py"

        log.info(f"test_confirm = {test_confirm}")

        test_confirm = str(Path(self.module_qc_nonelec_gui_dir) / test_confirm)
        log.info(f"test_confirm = {test_confirm}")

        self.confirm_module = machinery.SourceFileLoader(
            "confirm", test_confirm
        ).load_module()
        self.confirm_wid = self.confirm_module.ConfirmWindow(self)

        self.update_widget(self.confirm_wid)

    #        except(FileNotFoundError,self.confirm_module.NotSupportedError):
    except Exception:
        log.exception(traceback.format_exc())
        try:
            test_confirm = Path(self.module_qc_nonelec_gui_dir).joinpath(
                "GUI", "lib", "ConfirmWindow", "default_layout;py"
            )
            self.confirm_module = machinery.SourceFileLoader(
                "default_layout", test_confirm
            ).load_module()
            self.confirm_wid = self.confirm_module.ConfirmWindow(self)
        except FileNotFoundError:
            log.exception(traceback.format_exc())
            QMessageBox.warning(
                None,
                "Warning",
                'Module: "default_layout.py" is not found',
                QMessageBox.Ok,
            )
    self.update_widget(self.confirm_wid)


def upload_to_db_IV(self):
    if self.isPractice:
        self.terminate_message()
        return None

    if self.isOK_to_upload:
        try:
            self.local_save()

            protocol = self.custom_conf.get("localDB_web").get("protocol", "http")
            host = self.custom_conf.get("localDB_web").get("address", "127.0.0.1")
            port = self.custom_conf.get("localDB_web").get("port", "5000")

            log.info(
                f"attempting to upload TestRun to LocalDB on {protocol}://{host}:{port}/localdb/ ..."
            )

            res = requests.post(
                f"{protocol}://{host}:{port}/localdb/qc_uploader_post",
                json=[[self.testRun_sen]],
                timeout=60,
            )
            log.info("... posted! response = " + str(res.text))
            out = json.loads(str(res.text))
            log.info(pprint.pformat(out))

            sen_Obj_id = self.localDB.QC.result.find_one(
                {
                    "testType": "IV_MEASURE",
                    "stage": "BAREMODULERECEPTION",
                    "serialNumber": self.testRun_sen["serialNumber"],
                },
                sort=[("_id", pymongo.DESCENDING)],
                projection={"_id": 1},
            )
            self.testRun["results"]["LINK_TO_SENSOR_IV_TEST"] = self.json_rule(
                sen_Obj_id["_id"]
            )

            return True

        except Exception:
            log.exception(traceback.format_exc())
            log.exception(str(res.text))
            QMessageBox.warning(
                None,
                "Warning",
                "ERROR: failed to upload result to DB:\n\n======================\n\n"
                + str(res.text),
                QMessageBox.Ok,
            )
            return False
    else:
        QMessageBox.warning(
            None,
            "Warning",
            "you can not have prepared to upload",
            QMessageBox.Ok,
        )
        return False
